@@ -3,6 +3,7 @@
 
 #include <QObject>
 #include <QFont>
+#include <QString>
 
 #include "Radio.hpp"
 #include "models/IARURegions.hpp"
@@ -14,14 +15,12 @@
 class QSettings;
 class QWidget;
 class QAudioDeviceInfo;
-class QString;
 class QDir;
 class QNetworkAccessManager;
 class Bands;
-class FrequencyList_v2;
+class FrequencyList_v2_101;
 class StationList;
 class QStringListModel;
-class QHostAddress;
 class LotWUsers;
 class DecodeHighlightingModel;
 class LogBook;
@@ -101,6 +100,7 @@ public:
   QString my_grid () const;
   QString Field_Day_Exchange() const;
   QString RTTY_Exchange() const;
+  QString Contest_Name() const;
   void setEU_VHF_Contest();
   QFont text_font () const;
   QFont decoded_text_font () const;
@@ -113,6 +113,7 @@ public:
   bool id_after_73 () const;
   bool tx_QSY_allowed () const;
   bool spot_to_psk_reporter () const;
+  bool psk_reporter_tcpip () const;
   bool monitor_off_at_startup () const;
   bool monitor_last_used () const;
   bool log_as_RTTY () const;
@@ -151,6 +152,8 @@ public:
   void opCall (QString const&);
   QString udp_server_name () const;
   port_type udp_server_port () const;
+  QStringList udp_interface_names () const;
+  int udp_TTL () const;
   QString n1mm_server_name () const;
   port_type n1mm_server_port () const;
   bool valid_n1mm_info () const;
@@ -162,8 +165,8 @@ public:
   Bands * bands ();
   Bands const * bands () const;
   IARURegions::Region region () const;
-  FrequencyList_v2 * frequencies ();
-  FrequencyList_v2 const * frequencies () const;
+  FrequencyList_v2_101 * frequencies ();
+  FrequencyList_v2_101 const * frequencies () const;
   StationList * stations ();
   StationList const * stations () const;
   QStringListModel * macros ();
@@ -179,8 +182,16 @@ public:
   bool highlight_by_mode () const;
   bool highlight_only_fields () const;
   bool include_WAE_entities () const;
- 
-  enum class SpecialOperatingActivity {NONE, NA_VHF, EU_VHF, FIELD_DAY, RTTY, WW_DIGI, FOX, HOUND};
+  bool highlight_73 () const;
+  void setSpecial_Hound();
+  void setSpecial_Fox();
+  void setSpecial_None();
+  bool highlight_DXcall () const;
+  bool highlight_DXgrid () const;
+  bool Individual_Contest_Name() const;
+
+//                                      0       1      2         3       4       5      6     7        8
+  enum class SpecialOperatingActivity {NONE, NA_VHF, EU_VHF, FIELD_DAY, RTTY, WW_DIGI, FOX, HOUND, ARRL_DIGI};
   SpecialOperatingActivity special_op_id () const;
 
   struct CalibrationParams
@@ -259,6 +270,8 @@ public:
   // i.e. the transceiver is ready for use.
   Q_SLOT void sync_transceiver (bool force_signal = false, bool enforce_mode_and_split = false);
 
+  Q_SLOT void invalidate_audio_input_device (QString error);
+  Q_SLOT void invalidate_audio_output_device (QString error);
 
   //
   // These signals indicate a font has been selected and accepted for
@@ -270,8 +283,9 @@ public:
   //
   // This signal is emitted when the UDP server changes
   //
-  Q_SIGNAL void udp_server_changed (QString const& udp_server) const;
+  Q_SIGNAL void udp_server_changed (QString& udp_server_name, QStringList const& network_interfaces) const;
   Q_SIGNAL void udp_server_port_changed (port_type server_port) const;
+  Q_SIGNAL void udp_TTL_changed (int TTL) const;
   Q_SIGNAL void accept_udp_requests_changed (bool checked) const;
 
   // signal updates to decode highlighting
@@ -292,6 +306,12 @@ public:
   // the fault condition has been rectified or is transient.
   Q_SIGNAL void transceiver_failure (QString const& reason) const;
 
+  // signal announces audio devices are being enumerated
+  //
+  // As this can take some time, particularly on Linux, consumers
+  // might like to notify the user.
+  Q_SIGNAL void enumerating_audio_devices ();
+
 private:
   class impl;
   pimpl<impl> m_;
