@@ -49,7 +49,7 @@ auto OmniRigTransceiver::map_mode (OmniRig::RigParamX param) -> MODE
     {
       return FM;
     }
-  TRACE_CAT ("OmniRigTransceiver", "unrecognized mode");
+  CAT_ERROR ("unrecognized mode");
   throw_qstring (tr ("OmniRig: unrecognized mode"));
   return UNK;
 }
@@ -74,7 +74,9 @@ OmniRig::RigParamX OmniRigTransceiver::map_mode (MODE mode)
   return OmniRig::PM_SSB_U; // quieten compiler grumble
 }
 
-void OmniRigTransceiver::register_transceivers (TransceiverFactory::Transceivers * registry, int id1, int id2)
+void OmniRigTransceiver::register_transceivers (logger_type *,
+                                                TransceiverFactory::Transceivers * registry,
+                                                unsigned id1, unsigned id2)
 {
   (*registry)[OmniRig_transceiver_one_name] = TransceiverFactory::Capabilities {
     id1
@@ -94,10 +96,11 @@ void OmniRigTransceiver::register_transceivers (TransceiverFactory::Transceivers
   };
 }
 
-OmniRigTransceiver::OmniRigTransceiver (std::unique_ptr<TransceiverBase> wrapped,
+OmniRigTransceiver::OmniRigTransceiver (logger_type * the_logger,
+                                        std::unique_ptr<TransceiverBase> wrapped,
                                         RigNumber n, TransceiverFactory::PTTMethod ptt_type,
                                         QString const& ptt_port, QObject * parent)
-  : TransceiverBase {parent}
+  : TransceiverBase {the_logger, parent}
   , wrapped_ {std::move (wrapped)}
   , use_for_ptt_ {TransceiverFactory::PTT_method_CAT == ptt_type || ("CAT" == ptt_port && (TransceiverFactory::PTT_method_RTS == ptt_type || TransceiverFactory::PTT_method_DTR == ptt_type))}
   , ptt_type_ {ptt_type}
@@ -108,173 +111,131 @@ OmniRigTransceiver::OmniRigTransceiver (std::unique_ptr<TransceiverBase> wrapped
   , reversed_ {false}
 {
   CoInitializeEx (nullptr, 0 /*COINIT_APARTMENTTHREADED*/); // required because Qt only does this for GUI thread
+  CAT_TRACE ("constructed");
 }
 
 OmniRigTransceiver::~OmniRigTransceiver ()
 {
+  CAT_TRACE ("destroying");
   CoUninitialize ();
 }
 
-// returns false on time out
-bool OmniRigTransceiver::await_notification_with_timeout (int timeout)
-{
-  QEventLoop el;
-  connect (this, &OmniRigTransceiver::notified, &el, [&el] () {el.exit (1);});
-  QTimer::singleShot (timeout, Qt::CoarseTimer, &el, [&el] () {el.exit (0);});
-  return 1 == el.exec ();       // wait for notify or timer
-}
-
 int OmniRigTransceiver::do_start ()
 {
-  TRACE_CAT ("OmniRigTransceiver", "starting");
-
-  if (wrapped_) wrapped_->start (0);
-
-  omni_rig_.reset (new OmniRig::OmniRigX {this});
-  if (omni_rig_->isNull ())
+  CAT_TRACE ("starting");
+  try
     {
-      TRACE_CAT ("OmniRigTransceiver", "failed to start COM server");
-      throw_qstring (tr ("Failed to start OmniRig COM server"));
-    }
+      if (wrapped_) wrapped_->start (0);
 
-  // COM/OLE exceptions get signaled
-  connect (&*omni_rig_, SIGNAL (exception (int, QString, QString, QString)), this, SLOT (handle_COM_exception (int, QString, QString, QString)));
-
-  // IOmniRigXEvent interface signals
-  connect (&*omni_rig_, SIGNAL (VisibleChange ()), this, SLOT (handle_visible_change ()));
-  connect (&*omni_rig_, SIGNAL (RigTypeChange (int)), this, SLOT (handle_rig_type_change (int)));
-  connect (&*omni_rig_, SIGNAL (StatusChange (int)), this, SLOT (handle_status_change (int)));
-  connect (&*omni_rig_, SIGNAL (ParamsChange (int, int)), this, SLOT (handle_params_change (int, int)));
-  connect (&*omni_rig_
-           , SIGNAL (CustomReply (int, QVariant const&, QVariant const&))
-           , this, SLOT (handle_custom_reply (int, QVariant const&, QVariant const&)));
-
-  TRACE_CAT ("OmniRigTransceiver", "OmniRig s/w version:" << QString::number (omni_rig_->SoftwareVersion ()).toLocal8Bit ()
-             << "i/f version:" << QString::number (omni_rig_->InterfaceVersion ()).toLocal8Bit ());
-
-  // fetch the interface of the RigX CoClass and instantiate a proxy object
-  switch (rig_number_)
-    {
-    case One: rig_.reset (new OmniRig::RigX (omni_rig_->Rig1 ())); break;
-    case Two: rig_.reset (new OmniRig::RigX (omni_rig_->Rig2 ())); break;
-    }
-
-  Q_ASSERT (rig_);
-  Q_ASSERT (!rig_->isNull ());
-
-  // COM/OLE exceptions get signaled
-  connect (&*rig_, SIGNAL (exception (int, QString, QString, QString)), this, SLOT (handle_COM_exception (int, QString, QString, QString)));
-
-  offline_timer_.reset (new QTimer); // instantiate here as
-                                     // constructor runs in wrong
-                                     // thread
-  offline_timer_->setSingleShot (true);
-  connect (offline_timer_.data (), &QTimer::timeout, [this] () {offline ("Rig went offline");});
-
-  if (use_for_ptt_ && (TransceiverFactory::PTT_method_DTR == ptt_type_ || TransceiverFactory::PTT_method_RTS == ptt_type_))
-    {
-      // fetch the interface for the serial port if we need it for PTT
-      port_.reset (new OmniRig::PortBits (rig_->PortBits ()));
-
-      Q_ASSERT (port_);
-      Q_ASSERT (!port_->isNull ());
+      omni_rig_.reset (new OmniRig::OmniRigX {this});
+      if (omni_rig_->isNull ())
+        {
+          CAT_ERROR ("failed to start COM server");
+          throw_qstring (tr ("Failed to start OmniRig COM server"));
+        }
 
       // COM/OLE exceptions get signaled
-      connect (&*port_, SIGNAL (exception (int, QString, QString, QString)), this, SLOT (handle_COM_exception (int, QString, QString, QString)));
+      connect (&*omni_rig_, SIGNAL (exception (int, QString, QString, QString)), this, SLOT (handle_COM_exception (int, QString, QString, QString)));
 
-      TRACE_CAT ("OmniRigTransceiver", "OmniRig RTS state:" << port_->Rts ());
+      // IOmniRigXEvent interface signals
+      connect (&*omni_rig_, SIGNAL (VisibleChange ()), this, SLOT (handle_visible_change ()));
+      connect (&*omni_rig_, SIGNAL (RigTypeChange (int)), this, SLOT (handle_rig_type_change (int)));
+      connect (&*omni_rig_, SIGNAL (StatusChange (int)), this, SLOT (handle_status_change (int)));
+      connect (&*omni_rig_, SIGNAL (ParamsChange (int, int)), this, SLOT (handle_params_change (int, int)));
+      connect (&*omni_rig_
+               , SIGNAL (CustomReply (int, QVariant const&, QVariant const&))
+               , this, SLOT (handle_custom_reply (int, QVariant const&, QVariant const&)));
 
-      // remove locking because it doesn't seem to work properly
-      // if (!port_->Lock ()) // try to take exclusive use of the OmniRig serial port for PTT
-      //   {
-      //     TRACE_CAT ("OmniRigTransceiver", "Failed to get exclusive use of serial port for PTT from OmniRig");
-      //   }
+      CAT_INFO ("OmniRig s/w version: " << static_cast<quint16> (omni_rig_->SoftwareVersion () >> 16)
+                << '.' << static_cast<quint16> (omni_rig_->SoftwareVersion () & 0xffff)
+                << " i/f version: " << static_cast<int> (omni_rig_->InterfaceVersion () >> 8 & 0xff)
+                << '.' << static_cast<int> (omni_rig_->InterfaceVersion () && 0xff));
 
-      // start off so we don't accidentally key the radio
-      if (TransceiverFactory::PTT_method_DTR == ptt_type_)
+      // fetch the interface of the RigX CoClass and instantiate a proxy object
+      switch (rig_number_)
         {
-          port_->SetDtr (false);
+        case One: rig_.reset (new OmniRig::RigX (omni_rig_->Rig1 ())); break;
+        case Two: rig_.reset (new OmniRig::RigX (omni_rig_->Rig2 ())); break;
         }
-      else      // RTS
-        {
-          port_->SetRts (false);
-        }
-    }
 
-  rig_type_ = rig_->RigType ();
-  readable_params_ = rig_->ReadableParams ();
-  writable_params_ = rig_->WriteableParams ();
+      Q_ASSERT (rig_);
+      Q_ASSERT (!rig_->isNull ());
 
-  TRACE_CAT ("OmniRigTransceiver", QString {"OmniRig initial rig type: %1 readable params = 0x%2 writable params = 0x%3 for rig %4"}
-    .arg (rig_type_)
-    .arg (readable_params_, 8, 16, QChar ('0'))
-    .arg (writable_params_, 8, 16, QChar ('0'))
-    .arg (rig_number_).toLocal8Bit ());
-  for (int i = 0; i < 5; ++i)
-    {
-      if (OmniRig::ST_ONLINE == rig_->Status ())
+      // COM/OLE exceptions get signaled
+      connect (&*rig_, SIGNAL (exception (int, QString, QString, QString)), this, SLOT (handle_COM_exception (int, QString, QString, QString)));
+
+      offline_timer_.reset (new QTimer); // instantiate here as constructor runs in wrong thread
+      offline_timer_->setSingleShot (true);
+      connect (offline_timer_.data (), &QTimer::timeout, [this] () {offline ("Rig went offline");});
+
+      for (int i = 0; i < 5; ++i)
         {
-          break;
+          // leave some time for Omni-Rig to do its first poll
+          QThread::msleep (250);
+          if (OmniRig::ST_ONLINE == rig_->Status ())
+            {
+              break;
+            }
         }
-      await_notification_with_timeout (1000);
-    }
-  if (OmniRig::ST_ONLINE != rig_->Status ())
-    {
-      throw_qstring ("OmniRig: " + rig_->StatusStr ());
-    }
-  QThread::msleep (500);        // leave some time for Omni-Rig to get
-                                // the rig status for the 1st. time
-  auto f = rig_->GetRxFrequency ();
-  for (int i = 0; (f == 0) && (i < 5); ++i)
-    {
-      await_notification_with_timeout (1000);
-      f = rig_->GetRxFrequency ();
-    }
-  update_rx_frequency (f);
-  int resolution {0};
-  if (OmniRig::PM_UNKNOWN == rig_->Vfo ()
-      && (writable_params_ & (OmniRig::PM_VFOA | OmniRig::PM_VFOB))
-      == (OmniRig::PM_VFOA | OmniRig::PM_VFOB))
-    {
-      // start with VFO A (probably MAIN) on rigs that we
-      // can't query VFO but can set explicitly
-      rig_->SetVfo (OmniRig::PM_VFOA);
-    }
-  f = state ().frequency ();
-  if (f % 10) return resolution; // 1Hz resolution
-  auto test_frequency = f - f % 100 + 55;
-  if (OmniRig::PM_FREQ & writable_params_)
-    {
-      rig_->SetFreq (test_frequency);
-    }
-  else if (reversed_ && (OmniRig::PM_FREQB & writable_params_))
-    {
-      rig_->SetFreqB (test_frequency);
-    }
-  else if (!reversed_ && (OmniRig::PM_FREQA & writable_params_))
-    {
-      rig_->SetFreqA (test_frequency);
-    }
-  else
-    {
-      throw_qstring (tr ("OmniRig: don't know how to set rig frequency"));
-    }
-  if (!await_notification_with_timeout (1000))
-    {
-      TRACE_CAT ("OmniRigTransceiver", "do_start 1: wait timed out");
-      throw_qstring (tr ("OmniRig: timeout waiting for update from rig"));
-    }
-  switch (rig_->GetRxFrequency () - test_frequency)
-    {
-    case -5: resolution = -1; break;  // 10Hz truncated
-    case 5: resolution = 1; break;    // 10Hz rounded
-    case -15: resolution = -2; break; // 20Hz truncated
-    case -55: resolution = -2; break; // 100Hz truncated
-    case 45: resolution = 2; break;   // 100Hz rounded
-    }
-  if (1 == resolution)  // may be 20Hz rounded
-    {
-      test_frequency = f - f % 100 + 51;
+
+      if (OmniRig::ST_ONLINE != rig_->Status ())
+        {
+          CAT_ERROR ("rig not online");
+          throw_qstring ("OmniRig: " + rig_->StatusStr ());
+        }
+
+      if (use_for_ptt_ && (TransceiverFactory::PTT_method_DTR == ptt_type_ || TransceiverFactory::PTT_method_RTS == ptt_type_))
+        {
+          // fetch the interface for the serial port if we need it for PTT
+          port_.reset (new OmniRig::PortBits (rig_->PortBits ()));
+
+          Q_ASSERT (port_);
+          Q_ASSERT (!port_->isNull ());
+
+          // COM/OLE exceptions get signaled
+          connect (&*port_, SIGNAL (exception (int, QString, QString, QString)), this, SLOT (handle_COM_exception (int, QString, QString, QString)));
+
+          CAT_TRACE ("OmniRig RTS state: " << port_->Rts ());
+
+          // remove locking because it doesn't seem to work properly
+          // if (!port_->Lock ()) // try to take exclusive use of the OmniRig serial port for PTT
+          //   {
+          //     CAT_WARNING ("Failed to get exclusive use of serial port for PTT from OmniRig");
+          //   }
+
+          // start off so we don't accidentally key the radio
+          if (TransceiverFactory::PTT_method_DTR == ptt_type_)
+            {
+              port_->SetDtr (false);
+            }
+          else      // RTS
+            {
+              port_->SetRts (false);
+            }
+        }
+
+      rig_type_ = rig_->RigType ();
+      readable_params_ = rig_->ReadableParams ();
+      writable_params_ = rig_->WriteableParams ();
+
+      CAT_INFO (QString {"OmniRig initial rig type: %1 readable params=0x%2 writable params=0x%3 for rig %4"}
+         .arg (rig_type_)
+         .arg (readable_params_, 8, 16, QChar ('0'))
+         .arg (writable_params_, 8, 16, QChar ('0'))
+         .arg (rig_number_));
+      update_rx_frequency (rig_->GetRxFrequency ());
+      int resolution {0};
+      if (OmniRig::PM_UNKNOWN == rig_->Vfo ()
+          && (writable_params_ & (OmniRig::PM_VFOA | OmniRig::PM_VFOB))
+          == (OmniRig::PM_VFOA | OmniRig::PM_VFOB))
+        {
+          // start with VFO A (probably MAIN) on rigs that we
+          // can't query VFO but can set explicitly
+          rig_->SetVfo (OmniRig::PM_VFOA);
+        }
+      auto f = state ().frequency ();
+      if (f % 10) return resolution; // 1Hz resolution
+      auto test_frequency = f - f % 100 + 55;
       if (OmniRig::PM_FREQ & writable_params_)
         {
           rig_->SetFreq (test_frequency);
@@ -287,34 +248,78 @@ int OmniRigTransceiver::do_start ()
         {
           rig_->SetFreqA (test_frequency);
         }
-      if (!await_notification_with_timeout (2000))
+      else
         {
-          TRACE_CAT ("OmniRigTransceiver", "do_start 2: wait timed out");
-          throw_qstring (tr ("OmniRig: timeout waiting for update from rig"));
+          throw_qstring (tr ("OmniRig: don't know how to set rig frequency"));
         }
-      if (9 == rig_->GetRxFrequency () - test_frequency)
+      switch (rig_->GetRxFrequency () - test_frequency)
         {
-          resolution = 2;   // 20Hz rounded
+        case -5: resolution = -1; break;  // 10Hz truncated
+        case 5: resolution = 1; break;    // 10Hz rounded
+        case -15: resolution = -2; break; // 20Hz truncated
+        case -55: resolution = -2; break; // 100Hz truncated
+        case 45: resolution = 2; break;   // 100Hz rounded
         }
+      if (1 == resolution)  // may be 20Hz rounded
+        {
+          test_frequency = f - f % 100 + 51;
+          if (OmniRig::PM_FREQ & writable_params_)
+            {
+              rig_->SetFreq (test_frequency);
+            }
+          else if (reversed_ && (OmniRig::PM_FREQB & writable_params_))
+            {
+              rig_->SetFreqB (test_frequency);
+            }
+          else if (!reversed_ && (OmniRig::PM_FREQA & writable_params_))
+            {
+              rig_->SetFreqA (test_frequency);
+            }
+          if (9 == rig_->GetRxFrequency () - test_frequency)
+            {
+              resolution = 2;   // 20Hz rounded
+            }
+        }
+
+      // For OmniRig v1.19 or later we need a delay between GetRxFrequency () and SetFreq (f),
+      // otherwise rig QRG stays at f+55 Hz. 200 ms should do job for all modern transceivers.
+      // However, with very slow rigs, QRG may still stay at f+55 Hz. Such rigs should use v1.18.
+      // Due to the asynchronous nature of Omnirig commands, a better solution would be to implement
+      // an event handler for OmniRig's OnParamChange event and read the frequency inside that handler.
+
+      if (OmniRig::PM_FREQ & writable_params_)
+        {
+          QTimer::singleShot (200, [=] {
+              rig_->SetFreq (f);
+              });
+        }
+      else if (reversed_ && (OmniRig::PM_FREQB & writable_params_))
+        {
+          QTimer::singleShot (200, [=] {
+              rig_->SetFreqB (f);
+              });
+        }
+      else if (!reversed_ && (OmniRig::PM_FREQA & writable_params_))
+        {
+          QTimer::singleShot (200, [=] {
+              rig_->SetFreqA (f);
+              });
+        }
+      update_rx_frequency (f);
+      CAT_TRACE ("started");
+
+      return resolution;
     }
-  if (OmniRig::PM_FREQ & writable_params_)
+  catch (...)
     {
-      rig_->SetFreq (f);
+      CAT_TRACE ("start threw exception");
+      throw;
     }
-  else if (reversed_ && (OmniRig::PM_FREQB & writable_params_))
-    {
-      rig_->SetFreqB (f);
-    }
-  else if (!reversed_ && (OmniRig::PM_FREQA & writable_params_))
-    {
-      rig_->SetFreqA (f);
-    }
-  update_rx_frequency (f);
-  return resolution;
 }
 
 void OmniRigTransceiver::do_stop ()
 {
+  CAT_TRACE ("stopping");
   QThread::msleep (200);        // leave some time for pending
                                 // commands at the server end
 
@@ -334,6 +339,7 @@ void OmniRigTransceiver::do_stop ()
         {
           rig_->clear ();
           rig_.reset ();
+          CAT_TRACE ("rig_ reset");
         }
       omni_rig_->clear ();
       omni_rig_.reset ();
@@ -341,45 +347,45 @@ void OmniRigTransceiver::do_stop ()
   
   if (wrapped_) wrapped_->stop ();
 
-  TRACE_CAT ("OmniRigTransceiver", "stopped");
+  CAT_TRACE ("stopped");
 }
 
 void OmniRigTransceiver::handle_COM_exception (int code, QString source, QString desc, QString help)
 {
-  TRACE_CAT ("OmniRigTransceiver", QString::number (code) + " at " + source + ": " + desc + " (" + help + ')');
+  CAT_ERROR ((QString::number (code) + " at " + source + ": " + desc + " (" + help + ')'));
   throw_qstring (tr ("OmniRig COM/OLE error: %1 at %2: %3 (%4)").arg (QString::number (code)).arg (source). arg (desc). arg (help));
 }
 
 void OmniRigTransceiver::handle_visible_change ()
 {
   if (!omni_rig_ || omni_rig_->isNull ()) return;
-  TRACE_CAT ("OmniRigTransceiver", "visibility change: visibility =" << omni_rig_->DialogVisible ());
+  CAT_TRACE ("visibility change: visibility =" << omni_rig_->DialogVisible ());
 }
 
 void OmniRigTransceiver::handle_rig_type_change (int rig_number)
 {
-  TRACE_CAT ("OmniRigTransceiver", "rig type change: rig =" << rig_number);
+  CAT_TRACE ("rig type change: rig =" << rig_number);
   if (rig_number_ == rig_number)
     {
       if (!rig_ || rig_->isNull ()) return;
       readable_params_ = rig_->ReadableParams ();
       writable_params_ = rig_->WriteableParams ();
-      TRACE_CAT ("OmniRigTransceiver", QString {"rig type change to: %1 readable params = 0x%2 writable params = 0x%3 for rig %4"}
+      CAT_INFO (QString {"rig type change to: %1 readable params = 0x%2 writable params = 0x%3 for rig %4"}
         .arg (rig_->RigType ())
         .arg (readable_params_, 8, 16, QChar ('0'))
         .arg (writable_params_, 8, 16, QChar ('0'))
-        .arg (rig_number).toLocal8Bit ());
+        .arg (rig_number));
     }
 }
 
 void OmniRigTransceiver::handle_status_change (int rig_number)
 {
-  TRACE_CAT ("OmniRigTransceiver", QString {"status change for rig %1"}.arg (rig_number).toLocal8Bit ());
+  CAT_TRACE (QString {"status change for rig %1"}.arg (rig_number));
   if (rig_number_ == rig_number)
     {
       if (!rig_ || rig_->isNull ()) return;
-      auto const& status = rig_->StatusStr ().toLocal8Bit ();
-      TRACE_CAT ("OmniRigTransceiver", "OmniRig status change: new status = " << status);
+      auto const& status = rig_->StatusStr ();
+      CAT_TRACE ("OmniRig status change: new status = " << status);
       if (OmniRig::ST_ONLINE != rig_->Status ())
         {
           if (!offline_timer_->isActive ())
@@ -393,22 +399,21 @@ void OmniRigTransceiver::handle_status_change (int rig_number)
       else
         {
           offline_timer_->stop (); // good to go again
-          Q_EMIT notified ();
         }
       // else
       //   {
       //     update_rx_frequency (rig_->GetRxFrequency ());
       //     update_complete ();
-      //     TRACE_CAT ("OmniRigTransceiver", "frequency:" << state ().frequency ());
+      //     CAT_TRACE ("frequency:" << state ().frequency ());
       //   }
     }
 }
 
 void OmniRigTransceiver::handle_params_change (int rig_number, int params)
 {
-  TRACE_CAT ("OmniRigTransceiver", QString {"params change: params = 0x%1 for rig %2"}
+  CAT_TRACE (QString {"params change: params=0x%1 for rig %2"}
         .arg (params, 8, 16, QChar ('0'))
-        .arg (rig_number).toLocal8Bit ()
+        .arg (rig_number)
         << "state before:" << state ());
   if (rig_number_ == rig_number)
     {
@@ -419,7 +424,7 @@ void OmniRigTransceiver::handle_params_change (int rig_number, int params)
 
       if (params & OmniRig::PM_VFOAA)
         {
-          TRACE_CAT ("OmniRigTransceiver", "VFOAA");
+          CAT_TRACE ("VFOAA");
           update_split (false);
           reversed_ = false;
           update_rx_frequency (rig_->FreqA ());
@@ -427,7 +432,7 @@ void OmniRigTransceiver::handle_params_change (int rig_number, int params)
         }
       if (params & OmniRig::PM_VFOAB)
         {
-          TRACE_CAT ("OmniRigTransceiver", "VFOAB");
+          CAT_TRACE ("VFOAB");
           update_split (true);
           reversed_ = false;
           update_rx_frequency (rig_->FreqA ());
@@ -435,7 +440,7 @@ void OmniRigTransceiver::handle_params_change (int rig_number, int params)
         }
       if (params & OmniRig::PM_VFOBA)
         {
-          TRACE_CAT ("OmniRigTransceiver", "VFOBA");
+          CAT_TRACE ("VFOBA");
           update_split (true);
           reversed_ = true;
           update_other_frequency (rig_->FreqA ());
@@ -443,7 +448,7 @@ void OmniRigTransceiver::handle_params_change (int rig_number, int params)
         }
       if (params & OmniRig::PM_VFOBB)
         {
-          TRACE_CAT ("OmniRigTransceiver", "VFOBB");
+          CAT_TRACE ("VFOBB");
           update_split (false);
           reversed_ = true;
           update_other_frequency (rig_->FreqA ());
@@ -451,26 +456,25 @@ void OmniRigTransceiver::handle_params_change (int rig_number, int params)
         }
       if (params & OmniRig::PM_VFOA)
         {
-          TRACE_CAT ("OmniRigTransceiver", "VFOA");
+          CAT_TRACE ("VFOA");
           reversed_ = false;
           need_frequency = true;
         }
       if (params & OmniRig::PM_VFOB)
         {
-          TRACE_CAT ("OmniRigTransceiver", "VFOB");
+          CAT_TRACE ("VFOB");
           reversed_ = true;
           need_frequency = true;
         }
 
       if (params & OmniRig::PM_FREQ)
         {
-          TRACE_CAT ("OmniRigTransceiver", "FREQ");
           need_frequency = true;
         }
       if (params & OmniRig::PM_FREQA)
         {
           auto f = rig_->FreqA ();
-          TRACE_CAT ("OmniRigTransceiver", "FREQA = " << f);
+          CAT_TRACE ("FREQA = " << f);
           if (reversed_)
             {
               update_other_frequency (f);
@@ -483,7 +487,7 @@ void OmniRigTransceiver::handle_params_change (int rig_number, int params)
       if (params & OmniRig::PM_FREQB)
         {
           auto f = rig_->FreqB ();
-          TRACE_CAT ("OmniRigTransceiver", "FREQB = " << f);
+          CAT_TRACE ("FREQB = " << f);
           if (reversed_)
             {
               update_rx_frequency (f);
@@ -500,7 +504,7 @@ void OmniRigTransceiver::handle_params_change (int rig_number, int params)
               auto f = rig_->FreqA ();
               if (f)
                 {
-                  TRACE_CAT ("OmniRigTransceiver", "FREQA = " << f);
+                  CAT_TRACE ("FREQA = " << f);
                   if (reversed_)
                     {
                       update_other_frequency (f);
@@ -516,7 +520,7 @@ void OmniRigTransceiver::handle_params_change (int rig_number, int params)
               auto f = rig_->FreqB ();
               if (f)
                 {
-                  TRACE_CAT ("OmniRigTransceiver", "FREQB = " << f);
+                  CAT_TRACE ("FREQB = " << f);
                   if (reversed_)
                     {
                       update_rx_frequency (f);
@@ -532,35 +536,35 @@ void OmniRigTransceiver::handle_params_change (int rig_number, int params)
               auto f = rig_->Freq ();
               if (f)
                 {
-                  TRACE_CAT ("OmniRigTransceiver", "FREQ = " << f);
+                  CAT_TRACE ("FREQ = " << f);
                   update_rx_frequency (f);
                 }
             }
         }
       if (params & OmniRig::PM_PITCH)
         {
-          TRACE_CAT ("OmniRigTransceiver", "PITCH");
+          CAT_TRACE ("PITCH");
         }
       if (params & OmniRig::PM_RITOFFSET)
         {
-          TRACE_CAT ("OmniRigTransceiver", "RITOFFSET");
+          CAT_TRACE ("RITOFFSET");
         }
       if (params & OmniRig::PM_RIT0)
         {
-          TRACE_CAT ("OmniRigTransceiver", "RIT0");
+          CAT_TRACE ("RIT0");
         }
       if (params & OmniRig::PM_VFOEQUAL)
         {
           auto f = readable_params_ & OmniRig::PM_FREQA ? rig_->FreqA () : rig_->Freq ();
           auto m = map_mode (rig_->Mode ());
-          TRACE_CAT ("OmniRigTransceiver", QString {"VFOEQUAL f=%1 m=%2"}.arg (f).arg (m));
+          CAT_TRACE (QString {"VFOEQUAL f=%1 m=%2"}.arg (f).arg (m));
           update_rx_frequency (f);
           update_other_frequency (f);
           update_mode (m);
         }
       if (params & OmniRig::PM_VFOSWAP)
         {
-          TRACE_CAT ("OmniRigTransceiver", "VFOSWAP");
+          CAT_TRACE ("VFOSWAP");
           auto f = state ().tx_frequency ();
           update_other_frequency (state ().frequency ());
           update_rx_frequency (f);
@@ -568,78 +572,78 @@ void OmniRigTransceiver::handle_params_change (int rig_number, int params)
         }
       if (params & OmniRig::PM_SPLITON)
         {
-          TRACE_CAT ("OmniRigTransceiver", "SPLITON");
+          CAT_TRACE ("SPLITON");
           update_split (true);
         }
       if (params & OmniRig::PM_SPLITOFF)
         {
-          TRACE_CAT ("OmniRigTransceiver", "SPLITOFF");
+          CAT_TRACE ("SPLITOFF");
           update_split (false);
         }
       if (params & OmniRig::PM_RITON)
         {
-          TRACE_CAT ("OmniRigTransceiver", "RITON");
+          CAT_TRACE ("RITON");
         }
       if (params & OmniRig::PM_RITOFF)
         {
-          TRACE_CAT ("OmniRigTransceiver", "RITOFF");
+          CAT_TRACE ("RITOFF");
         }
       if (params & OmniRig::PM_XITON)
         {
-          TRACE_CAT ("OmniRigTransceiver", "XITON");
+          CAT_TRACE ("XITON");
         }
       if (params & OmniRig::PM_XITOFF)
         {
-          TRACE_CAT ("OmniRigTransceiver", "XITOFF");
+          CAT_TRACE ("XITOFF");
         }
       if (params & OmniRig::PM_RX)
         {
-          TRACE_CAT ("OmniRigTransceiver", "RX");
+          CAT_TRACE ("RX");
           update_PTT (false);
         }
       if (params & OmniRig::PM_TX)
         {
-          TRACE_CAT ("OmniRigTransceiver", "TX");
+          CAT_TRACE ("TX");
           update_PTT ();
         }
       if (params & OmniRig::PM_CW_U)
         {
-          TRACE_CAT ("OmniRigTransceiver", "CW-R");
+          CAT_TRACE ("CW-R");
           update_mode (CW_R);
         }
       if (params & OmniRig::PM_CW_L)
         {
-          TRACE_CAT ("OmniRigTransceiver", "CW");
+          CAT_TRACE ("CW");
           update_mode (CW);
         }
       if (params & OmniRig::PM_SSB_U)
         {
-          TRACE_CAT ("OmniRigTransceiver", "USB");
+          CAT_TRACE ("USB");
           update_mode (USB);
         }
       if (params & OmniRig::PM_SSB_L)
         {
-          TRACE_CAT ("OmniRigTransceiver", "LSB");
+          CAT_TRACE ("LSB");
           update_mode (LSB);
         }
       if (params & OmniRig::PM_DIG_U)
         {
-          TRACE_CAT ("OmniRigTransceiver", "DATA-U");
+          CAT_TRACE ("DATA-U");
           update_mode (DIG_U);
         }
       if (params & OmniRig::PM_DIG_L)
         {
-          TRACE_CAT ("OmniRigTransceiver", "DATA-L");
+          CAT_TRACE ("DATA-L");
           update_mode (DIG_L);
         }
       if (params & OmniRig::PM_AM)
         {
-          TRACE_CAT ("OmniRigTransceiver", "AM");
+          CAT_TRACE ("AM");
           update_mode (AM);
         }
       if (params & OmniRig::PM_FM)
         {
-          TRACE_CAT ("OmniRigTransceiver", "FM");
+          CAT_TRACE ("FM");
           update_mode (FM);
         }
 
@@ -648,9 +652,8 @@ void OmniRigTransceiver::handle_params_change (int rig_number, int params)
           update_complete ();
           send_update_signal_ = false;
         }
-      TRACE_CAT ("OmniRigTransceiver", "OmniRig params change: state after:" << state ());
+      CAT_TRACE ("OmniRig params change: state after:" << state ());
     }
-  Q_EMIT notified ();
 }
 
 void OmniRigTransceiver::handle_custom_reply (int rig_number, QVariant const& command, QVariant const& reply)
@@ -661,19 +664,19 @@ void OmniRigTransceiver::handle_custom_reply (int rig_number, QVariant const& co
   if (rig_number_ == rig_number)
     {
       if (!rig_ || rig_->isNull ()) return;
-      TRACE_CAT ("OmniRigTransceiver", "custom command" << command.toString ().toLocal8Bit ()
-                 << "with reply" << reply.toString ().toLocal8Bit ()
-                 << QString ("for rig %1").arg (rig_number).toLocal8Bit ());
-      TRACE_CAT ("OmniRigTransceiver", "rig number:" << rig_number_ << ':' << state ());
+      CAT_TRACE ("custom command" << command.toString ()
+                 << "with reply" << reply.toString ()
+                 << QString ("for rig %1").arg (rig_number));
+      CAT_TRACE ("rig number:" << rig_number_ << ':' << state ());
     }
 }
 
 void OmniRigTransceiver::do_ptt (bool on)
 {
-  TRACE_CAT ("OmniRigTransceiver", on << state ());
+  CAT_TRACE (on << state ());
   if (use_for_ptt_ && TransceiverFactory::PTT_method_CAT == ptt_type_)
     {
-      TRACE_CAT ("OmniRigTransceiver", "set PTT");
+      CAT_TRACE ("set PTT");
       if (rig_ && !rig_->isNull ())
         {
           rig_->SetTx (on ? OmniRig::PM_TX : OmniRig::PM_RX);
@@ -685,18 +688,18 @@ void OmniRigTransceiver::do_ptt (bool on)
         {
           if (TransceiverFactory::PTT_method_RTS == ptt_type_)
             {
-              TRACE_CAT ("OmniRigTransceiver", "set RTS");
+              CAT_TRACE ("set RTS");
               port_->SetRts (on);
             }
           else      // "DTR"
             {
-              TRACE_CAT ("OmniRigTransceiver", "set DTR");
+              CAT_TRACE ("set DTR");
               port_->SetDtr (on);
             }
         }
       else if (wrapped_)
         {
-          TRACE_CAT ("OmniRigTransceiver", "set PTT using basic transceiver");
+          CAT_TRACE ("set PTT using basic transceiver");
           TransceiverState new_state {wrapped_->state ()};
           new_state.ptt (on);
           wrapped_->set (new_state, 0);
@@ -707,7 +710,7 @@ void OmniRigTransceiver::do_ptt (bool on)
 
 void OmniRigTransceiver::do_frequency (Frequency f, MODE m, bool /*no_ignore*/)
 {
-  TRACE_CAT ("OmniRigTransceiver", f << state ());
+  CAT_TRACE (f << ' ' << state ());
   if (!rig_ || rig_->isNull ()) return;
   if (UNK != m)
     {
@@ -736,7 +739,7 @@ void OmniRigTransceiver::do_frequency (Frequency f, MODE m, bool /*no_ignore*/)
 
 void OmniRigTransceiver::do_tx_frequency (Frequency tx, MODE m, bool /*no_ignore*/)
 {
-  TRACE_CAT ("OmniRigTransceiver", tx << state ());
+  CAT_TRACE (tx << ' ' << state ());
   if (!rig_ || rig_->isNull ()) return;
   bool split {tx != 0};
   if (split)
@@ -766,14 +769,14 @@ void OmniRigTransceiver::do_tx_frequency (Frequency tx, MODE m, bool /*no_ignore
                 }
             }
         }
-      TRACE_CAT ("OmniRigTransceiver", "set SPLIT mode on");
+      CAT_TRACE ("set SPLIT mode on");
       rig_->SetSplitMode (state ().frequency (), tx);
       update_other_frequency (tx);
       update_split (true);
     }
   else
     {
-      TRACE_CAT ("OmniRigTransceiver", "set SPLIT mode off");
+      CAT_TRACE ("set SPLIT mode off");
       rig_->SetSimplexMode (state ().frequency ());
       update_split (false);
     }
@@ -788,7 +791,7 @@ void OmniRigTransceiver::do_tx_frequency (Frequency tx, MODE m, bool /*no_ignore
     }
   if (!((OmniRig::PM_VFOAB | OmniRig::PM_VFOBA | OmniRig::PM_SPLITON) & readable_params_))
     {
-      TRACE_CAT ("OmniRigTransceiver", "setting SPLIT manually");
+      CAT_TRACE ("setting SPLIT manually");
       update_split (split); // we can't read it so just set and
       // hope op doesn't change it
       notify = true;
@@ -801,7 +804,7 @@ void OmniRigTransceiver::do_tx_frequency (Frequency tx, MODE m, bool /*no_ignore
 
 void OmniRigTransceiver::do_mode (MODE mode)
 {
-  TRACE_CAT ("OmniRigTransceiver", mode << state ());
+  CAT_TRACE (mode << ' ' << state ());
   if (!rig_ || rig_->isNull ()) return;
   // TODO: G4WJS OmniRig doesn't seem to have any capability of tracking/setting VFO B mode
   auto mapped = map_mode (mode);
