@@ -2,12 +2,13 @@ module packjt77
 
 ! These variables are accessible from outside via "use packjt77":
   parameter (MAXHASH=1000,MAXRECENT=10)
-  character (len=13), dimension(1:1024) ::  calls10=''
-  character (len=13), dimension(1:4096) ::  calls12=''
+  character (len=13), dimension(0:1023) ::  calls10=''
+  character (len=13), dimension(0:4095) ::  calls12=''
   character (len=13), dimension(1:MAXHASH) :: calls22=''
   character (len=13), dimension(1:MAXRECENT) :: recent_calls=''
   character (len=13) :: mycall13=''
   character (len=13) :: dxcall13=''
+  character (len=6)  :: dxbase=''
   integer, dimension(1:MAXHASH) :: ihash22=-1
   integer :: nzhash=0
   integer n28a,n28b
@@ -19,7 +20,7 @@ subroutine hash10(n10,c13)
   character*13 c13
 
   c13='<...>'
-  if(n10.lt.1 .or. n10.gt.1024) return
+  if(n10.lt.0 .or. n10.gt.1023) return
   if(len(trim(calls10(n10))).gt.0) then
      c13=calls10(n10)
      c13='<'//trim(c13)//'>'
@@ -33,7 +34,7 @@ subroutine hash12(n12,c13)
   character*13 c13
   
   c13='<...>'
-  if(n12.lt.1 .or. n12.gt.4096) return
+  if(n12.lt.0 .or. n12.gt.4095) return
   if(len(trim(calls12(n12))).gt.0) then
      c13=calls12(n12)
      c13='<'//trim(c13)//'>'
@@ -90,10 +91,10 @@ subroutine save_hash_call(c13,n10,n12,n22)
   if(len(trim(cw)) .lt. 3) return
 
   n10=ihashcall(cw,10)
-  if(n10.ge.1 .and. n10 .le. 1024 .and. cw.ne.mycall13) calls10(n10)=cw
+  if(n10.ge.0 .and. n10 .le. 1023 .and. cw.ne.mycall13) calls10(n10)=cw
 
   n12=ihashcall(cw,12)
-  if(n12.ge.1 .and. n12 .le. 4096 .and. cw.ne.mycall13) calls12(n12)=cw
+  if(n12.ge.0 .and. n12 .le. 4095 .and. cw.ne.mycall13) calls12(n12)=cw
 
   n22=ihashcall(cw,22)
   if(any(ihash22.eq.n22)) then   ! If entry exists, make sure callsign is the most recently received one 
@@ -124,12 +125,19 @@ subroutine pack77(msg0,i3,n3,c77)
   integer ntel(3)
 
   msg=msg0
-  if(i3.eq.0 .and. n3.eq.5) go to 5
+  if(msg(1:3).eq.'$DX') then
+     i1=index(msg,' ')
+     msg=trim(dxbase)//' '//msg(i1+1:)
+  endif
+
+  i3_hint=i3
+  n3_hint=n3
+  i3=-1
+  n3=-1
+  if(i3_hint.eq.0 .and. n3_hint.eq.5) go to 5
 
 ! Convert msg to upper case; collapse multiple blanks; parse into words.
   call split77(msg,nwords,nw,w)
-  i3=-1
-  n3=-1
   if(msg(1:3).eq.'CQ ' .or. msg(1:3).eq.'DE ' .or. msg(1:4).eq.'QRZ ') go to 100
 
 ! Check 0.1 (DXpedition mode)
@@ -160,7 +168,7 @@ subroutine pack77(msg0,i3,n3,c77)
      go to 900
   endif
 
-100 call pack77_06(nwords,w,i3,n3,c77)
+100 call pack77_06(nwords,w,i3,n3,c77,i3_hint,n3_hint)
   if(i3.ge.0) go to 900
 
 ! Check Type 1 (Standard 77-bit message) or Type 2, with optional "/P"
@@ -196,14 +204,14 @@ subroutine unpack77(c77,nrx,msg,unpk77_success)
 ! the value of nrx is used to decide when mycall13 or dxcall13 should
 ! be used in place of a callsign from the hashtable
 !
-  parameter (NSEC=85)      !Number of ARRL Sections
-  parameter (NUSCAN=65)    !Number of US states and Canadian provinces
+  parameter (NSEC=86)      !Number of ARRL Sections
+  parameter (NUSCAN=171)   !Number of States and Provinces
   parameter (MAXGRID4=32400)
   integer*8 n58
   integer ntel(3)
   character*77 c77
   character*37 msg
-  character*13 call_1,call_2,call_3
+  character*13 call_1,call_2,call_3,call_1a
   character*13 mycall13_0,dxcall13_0
   character*11 c11
   character*3 crpt,cntx,cpfx
@@ -214,21 +222,21 @@ subroutine unpack77(c77,nrx,msg,unpk77_success)
   character*38 c
   character*36 a2
   integer hashmy10,hashmy12,hashmy22,hashdx10,hashdx12,hashdx22
-  logical unpk28_success,unpk77_success
+  logical unpk28_success,unpk77_success,unpkg4_success
   logical dxcall13_set,mycall13_set
 
   data a2/'0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ'/,nzzz/46656/
   data c/' 0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ/'/
   data csec/                                                         &
        "AB ","AK ","AL ","AR ","AZ ","BC ","CO ","CT ","DE ","EB ",  &       
-       "EMA","ENY","EPA","EWA","GA ","GTA","IA ","ID ","IL ","IN ",  &       
-       "KS ","KY ","LA ","LAX","MAR","MB ","MDC","ME ","MI ","MN ",  &       
+       "EMA","ENY","EPA","EWA","GA ","GH ","IA ","ID ","IL ","IN ",  &
+       "KS ","KY ","LA ","LAX","NS ","MB ","MDC","ME ","MI ","MN ",  &
        "MO ","MS ","MT ","NC ","ND ","NE ","NFL","NH ","NL ","NLI",  &       
-       "NM ","NNJ","NNY","NT ","NTX","NV ","OH ","OK ","ONE","ONN",  &       
+       "NM ","NNJ","NNY","TER","NTX","NV ","OH ","OK ","ONE","ONN",  &
        "ONS","OR ","ORG","PAC","PR ","QC ","RI ","SB ","SC ","SCV",  &       
        "SD ","SDG","SF ","SFL","SJV","SK ","SNJ","STX","SV ","TN ",  &       
        "UT ","VA ","VI ","VT ","WCF","WI ","WMA","WNY","WPA","WTX",  &       
-       "WV ","WWA","WY ","DX ","PE "/
+       "WV ","WWA","WY ","DX ","PE ","NB "/
   data cmult/                                                        &
        "AL ","AK ","AZ ","AR ","CA ","CO ","CT ","DE ","FL ","GA ",  &
        "HI ","ID ","IL ","IN ","IA ","KS ","KY ","LA ","ME ","MD ",  &
@@ -236,7 +244,18 @@ subroutine unpack77(c77,nrx,msg,unpk77_success)
        "NM ","NY ","NC ","ND ","OH ","OK ","OR ","PA ","RI ","SC ",  &
        "SD ","TN ","TX ","UT ","VT ","VA ","WA ","WV ","WI ","WY ",  &
        "NB ","NS ","QC ","ON ","MB ","SK ","AB ","BC ","NWT","NF ",  &
-       "LB ","NU ","YT ","PEI","DC "/
+       "LB ","NU ","YT ","PEI","DC ","DR ","FR ","GD ","GR ","OV ",  &
+       "ZH ","ZL ","X01","X02","X03","X04","X05","X06","X07","X08",  &
+       "X09","X10","X11","X12","X13","X14","X15","X16","X17","X18",  &
+       "X19","X20","X21","X22","X23","X24","X25","X26","X27","X28",  &
+       "X29","X30","X31","X32","X33","X34","X35","X36","X37","X38",  &
+       "X39","X40","X41","X42","X43","X44","X45","X46","X47","X48",  &
+       "X49","X50","X51","X52","X53","X54","X55","X56","X57","X58",  &
+       "X59","X60","X61","X62","X63","X64","X65","X66","X67","X68",  &
+       "X69","X70","X71","X72","X73","X74","X75","X76","X77","X78",  &
+       "X79","X80","X81","X82","X83","X84","X85","X86","X87","X88",  &
+       "X89","X90","X91","X92","X93","X94","X95","X96","X97","X98",  &
+       "X99"/
   data dxcall13_set/.false./
   data mycall13_set/.false./
   data mycall13_0/''/
@@ -276,11 +295,16 @@ subroutine unpack77(c77,nrx,msg,unpk77_success)
 
   read(c77(72:77),'(2b3)') n3,i3
   msg=repeat(' ',37)
+
   if(i3.eq.0 .and. n3.eq.0) then
 ! 0.0  Free text
      call unpacktext77(c77(1:71),msg(1:13))
      msg(14:)='                        '
      msg=adjustl(msg)
+     if(msg(1:1).eq.' ') then
+        unpk77_success=.false.
+        return
+     endif
 
   else if(i3.eq.0 .and. n3.eq.1) then
 ! 0.1  K1ABC RR73; W9XYZ <KH1/KH7Z> -11   28 28 10 5       71   DXpedition Mode
@@ -346,38 +370,32 @@ subroutine unpack77(c77,nrx,msg,unpk77_success)
      msg=adjustl(msg)
 
   else if(i3.eq.0 .and. n3.eq.6) then
-     read(c77(50:50),'(b1)') j2a
-     j2b=0
-     if(j2a.eq.0) read(c77(49:49),'(b1)') j2b
-     j2=2*j2a+j2b
-     if(j2.eq.0) then
+     read(c77(49:50),'(2b1)') j2a,j2b
+     itype=2
+     if(j2b.eq.0 .and. j2a.eq.0) itype=1
+     if(j2b.eq.0 .and. j2a.eq.1) itype=3
+     if(itype.eq.1) then
 ! WSPR Type 1
         read(c77,2010) n28,igrid4,idbm
 2010    format(b28.28,b15.15,b5.5)
         idbm=nint(idbm*10.0/3.0)
         call unpack28(n28,call_1,unpk28_success) 
         if(.not.unpk28_success) unpk77_success=.false.
-        call to_grid4(igrid4,grid4)
+        call to_grid4(igrid4,grid4,unpkg4_success)
+        if(.not.unpkg4_success) unpk77_success=.false.
         write(crpt,'(i3)') idbm
         msg=trim(call_1)//' '//grid4//' '//trim(adjustl(crpt))
+        if (unpk77_success) call save_hash_call(call_1,n10,n12,n22) !### Is this OK here? ###
 
-     else if(j2.eq.1) then
+     else if(itype.eq.2) then
 ! WSPR Type 2
-        read(c77,2030) n28,igrid6
-2030    format(b22.22,b25.25)
-        call unpack28(n28,call_1,unpk28_success) 
-        if(.not.unpk28_success) unpk77_success=.false.
-        call to_grid6(igrid6,grid6)
-        msg=trim(call_1)//' '//grid6
-
-     else if(j2.eq.2) then
-! WSPR Type 3
         read(c77,2020) n28,npfx,idbm
 2020    format(b28.28,b16.16,b5.5)
         idbm=nint(idbm*10.0/3.0)        
         call unpack28(n28,call_1,unpk28_success) 
         if(.not.unpk28_success) unpk77_success=.false.
         write(crpt,'(i3)') idbm
+        cpfx='   '
         if(npfx.lt.nzzz) then
 ! Prefix
            do i=3,1,-1
@@ -387,10 +405,11 @@ subroutine unpack77(c77,nrx,msg,unpk77_success)
               if(npfx.eq.0) exit
            enddo
            msg=trim(adjustl(cpfx))//'/'//trim(call_1)//' '//trim(adjustl(crpt))
+           call_1a=trim(adjustl(cpfx))//'/'//trim(call_1)
+           call save_hash_call(call_1a,n10,n12,n22)  !### Is this OK here? ###
         else
 ! Suffix
            npfx=npfx-nzzz
-           cpfx='   '
            if(npfx.le.35) then
               cpfx(1:1)=a2(npfx+1:npfx+1)
            else if(npfx.gt.35 .and. npfx.le.1295) then
@@ -405,10 +424,25 @@ subroutine unpack77(c77,nrx,msg,unpk77_success)
               return
            endif
            msg=trim(call_1)//'/'//trim(adjustl(cpfx))//' '//trim(adjustl(crpt))
+           call_1a=trim(call_1)//'/'//trim(adjustl(cpfx))
+           call save_hash_call(call_1a,n10,n12,n22)  !### Is this OK here? ###
         endif
-
+        
+     else if(itype.eq.3) then
+! WSPR Type 3
+        read(c77,2030) n22,igrid6
+2030    format(b22.22,b25.25)
+        n28=n22+2063592
+        call unpack28(n28,call_1,unpk28_success) 
+        if(.not.unpk28_success) unpk77_success=.false.
+        call to_grid(igrid6,grid6,unpkg4_success)
+        if(.not.unpkg4_success) unpk77_success=.false.
+        msg=trim(call_1)//' '//grid6
      endif
-     
+  
+  else if(i3.eq.0 .and. n3.gt.6) then
+     unpk77_success=.false.
+   
   else if(i3.eq.1 .or. i3.eq.2) then
 ! Type 1 (standard message) or Type 2 ("/P" form for EU VHF contest)
      read(c77,1000) n28a,ipa,n28b,ipb,ir,igrid4,i3
@@ -435,7 +469,8 @@ subroutine unpack77(c77,nrx,msg,unpk77_success)
         if(i.ge.4) call add_call_to_recent_calls(call_2)
      endif
      if(igrid4.le.MAXGRID4) then
-        call to_grid4(igrid4,grid4)
+        call to_grid4(igrid4,grid4,unpkg4_success)
+        if(.not.unpkg4_success) unpk77_success=.false.
         if(ir.eq.0) msg=trim(call_1)//' '//trim(call_2)//' '//grid4
         if(ir.eq.1) msg=trim(call_1)//' '//trim(call_2)//' R '//grid4
         if(msg(1:3).eq.'CQ ' .and. ir.eq.1) unpk77_success=.false.
@@ -446,7 +481,9 @@ subroutine unpack77(c77,nrx,msg,unpk77_success)
         if(irpt.eq.3) msg=trim(call_1)//' '//trim(call_2)//' RR73'
         if(irpt.eq.4) msg=trim(call_1)//' '//trim(call_2)//' 73'
         if(irpt.ge.5) then
-           write(crpt,'(i3.2)') irpt-35
+           isnr=irpt-35
+           if(isnr.gt.50) isnr=isnr-101
+           write(crpt,'(i3.2)') isnr
            if(crpt(1:1).eq.' ') crpt(1:1)='+'
            if(ir.eq.0) msg=trim(call_1)//' '//trim(call_2)//' '//crpt
            if(ir.eq.1) msg=trim(call_1)//' '//trim(call_2)//' R'//crpt
@@ -550,7 +587,7 @@ subroutine unpack77(c77,nrx,msg,unpk77_success)
      nrs=52+irpt
      write(cexch,1022) nrs,iserial
 1022 format(i2,i4.4)
-     call to_grid6(igrid6,grid6)
+     call to_grid6(igrid6,grid6,unpk77_success)
      if(ir.eq.0) msg=trim(call_1)//' '//trim(call_2)//' '//cexch//' '//grid6
      if(ir.eq.1) msg=trim(call_1)//' '//trim(call_2)//' R '//cexch//' '//grid6
 
@@ -796,24 +833,26 @@ subroutine split77(msg,nwords,nw,w)
      endif
      j=j+1                                      !Index in msg
      n=n+1                                      !Index in word
+     if(c.ge.'a' .and. c.le.'z') c=char(ichar(c)-32)  !Force upper case
      msg(j:j)=c
-     if(c.ge.'a' .and. c.le.'z') msg(j:j)=char(ichar(c)-32)  !Force upper case
      if(n.le.13) w(k)(n:n)=c                    !Copy character c into word
      c0=c
   enddo
   iz=j                                          !Message length
   nwords=k                                      !Number of words in msg
   if(nwords.le.0) go to 900
-  nw(k)=len(trim(w(k)))
+  do i=1,nwords
+     nw(i)=len(trim(w(i)))
+  enddo
   msg(iz+1:)='                                     '
   if(nwords.lt.3) go to 900
   call chkcall(w(3),bcall_1,ok1)
   if(ok1 .and. w(1)(1:3).eq.'CQ ') then
      w(1)='CQ_'//w(2)(1:10)             !Make "CQ " into "CQ_"
-     w(2:12)=w(3:13)                    !Move all remeining words down by one
+     w(2:12)=w(3:13)                    !Move all remaining words down by one
      nwords=nwords-1
   endif
-  
+
 900 return
 end subroutine split77
 
@@ -864,7 +903,7 @@ subroutine pack77_03(nwords,w,i3,n3,c77)
 ! Check 0.3 and 0.4 (ARRL Field Day exchange)
 ! Example message:  WA9XYZ KA1ABC R 16A EMA       28 28 1 4 3 7    71  
 
-  parameter (NSEC=85)      !Number of ARRL Sections
+  parameter (NSEC=86)      !Number of ARRL Sections
   character*13 w(19)
   character*77 c77
   character*6 bcall_1,bcall_2
@@ -872,14 +911,14 @@ subroutine pack77_03(nwords,w,i3,n3,c77)
   logical ok1,ok2
   data csec/                                                         &
        "AB ","AK ","AL ","AR ","AZ ","BC ","CO ","CT ","DE ","EB ",  &       
-       "EMA","ENY","EPA","EWA","GA ","GTA","IA ","ID ","IL ","IN ",  &       
-       "KS ","KY ","LA ","LAX","MAR","MB ","MDC","ME ","MI ","MN ",  &       
+       "EMA","ENY","EPA","EWA","GA ","GH ","IA ","ID ","IL ","IN ",  &
+       "KS ","KY ","LA ","LAX","NS ","MB ","MDC","ME ","MI ","MN ",  &
        "MO ","MS ","MT ","NC ","ND ","NE ","NFL","NH ","NL ","NLI",  &       
-       "NM ","NNJ","NNY","NT ","NTX","NV ","OH ","OK ","ONE","ONN",  &       
+       "NM ","NNJ","NNY","TER","NTX","NV ","OH ","OK ","ONE","ONN",  &
        "ONS","OR ","ORG","PAC","PR ","QC ","RI ","SB ","SC ","SCV",  &       
        "SD ","SDG","SF ","SFL","SJV","SK ","SNJ","STX","SV ","TN ",  &       
        "UT ","VA ","VI ","VT ","WCF","WI ","WMA","WNY","WPA","WTX",  &       
-       "WV ","WWA","WY ","DX ","PE "/
+       "WV ","WWA","WY ","DX ","PE ","NB "/
 
   if(nwords.lt.4 .or. nwords.gt.5) return  
   call chkcall(w(1),bcall_1,ok1)
@@ -897,7 +936,7 @@ subroutine pack77_03(nwords,w,i3,n3,c77)
   
   ntx=-1
   j=len(trim(w(nwords-1)))-1
-  read(w(nwords-1)(1:j),*,err=1) ntx                !Number of transmitters
+  read(w(nwords-1)(1:j),*,err=1,end=1) ntx          !Number of transmitters
 1 if(ntx.lt.1 .or. ntx.gt.32) return
   nclass=ichar(w(nwords-1)(j+1:j+1))-ichar('A')
   
@@ -925,7 +964,7 @@ subroutine pack77_03(nwords,w,i3,n3,c77)
 end subroutine pack77_03
 
 
-subroutine pack77_06(nwords,w,i3,n3,c77)
+subroutine pack77_06(nwords,w,i3,n3,c77,i3_hint,n3_hint)
 
   character*13 w(19)
   character*77 c77
@@ -942,13 +981,14 @@ subroutine pack77_06(nwords,w,i3,n3,c77)
        grid4(3:3).ge.'0' .and. grid4(3:3).le.'9' .and.               &
        grid4(4:4).ge.'0' .and. grid4(4:4).le.'9'
 
-  is_grid6(grid6)=len(trim(grid6)).eq.6 .and.                        &
+  is_grid6(grid6)=(len(trim(grid6)).eq.6.or.len(trim(grid6)).eq.4).and. &
        grid6(1:1).ge.'A' .and. grid6(1:1).le.'R' .and.               &
        grid6(2:2).ge.'A' .and. grid6(2:2).le.'R' .and.               &
        grid6(3:3).ge.'0' .and. grid6(3:3).le.'9' .and.               &
        grid6(4:4).ge.'0' .and. grid6(4:4).le.'9' .and.               &
-       grid6(5:5).ge.'A' .and. grid6(5:5).le.'X' .and.               &
-       grid6(6:6).ge.'A' .and. grid6(6:6).le.'X'
+       (len(trim(grid6)).eq.4.or.                                    &
+       (grid6(5:5).ge.'A' .and. grid6(5:5).le.'X' .and.              &
+       grid6(6:6).ge.'A' .and. grid6(6:6).le.'X'))
 
   is_digit(c)=c.ge.'0' .and. c.le.'9'
 
@@ -1020,22 +1060,32 @@ subroutine pack77_06(nwords,w,i3,n3,c77)
      go to 900
   endif
 
-  if(nwords.eq.2 .and. m1.ge.5 .and. m1.le.12 .and. m2.le.6) then
+  if(i3_hint.eq.0.and.n3_hint.eq.6.and.nwords.eq.2 .and. m1.ge.5  &
+       .and. m1.le.12 .and. m2.le.6) then
 ! WSPR Type 3
+
+     !n3_hint=6 and i3_hint=0 is a hint that the caller wanted a
+     !50-bit encoding rather than the possible alternative n3=4 77-bit
+     !encoding
      if(index(w(1),'<').lt.1 .or. index(w(1),'>').lt.1) go to 900
      grid6=w(2)(1:6)
      if(.not.is_grid6(grid6)) go to 900
      i3=0
      n3=6
      call pack28(w(1),n28)
-     k1=(ichar(grid6(1:1))-ichar('A'))*18*10*10*24*24
-     k2=(ichar(grid6(2:2))-ichar('A'))*10*10*24*24
-     k3=(ichar(grid6(3:3))-ichar('0'))*10*24*24
-     k4=(ichar(grid6(4:4))-ichar('0'))*24*24
-     k5=(ichar(grid6(5:5))-ichar('A'))*24
-     k6=(ichar(grid6(6:6))-ichar('A'))
-     igrid6=k1+k2+k3+k4+k5+k6
-     write(c77,1030) n28,igrid6,2,0,n3,i3
+     n22=n28-2063592
+     k1=(ichar(grid6(1:1))-ichar('A'))*18*10*10*25*25
+     k2=(ichar(grid6(2:2))-ichar('A'))*10*10*25*25
+     k3=(ichar(grid6(3:3))-ichar('0'))*10*25*25
+     k4=(ichar(grid6(4:4))-ichar('0'))*25*25
+     if (grid6(5:6).eq.'  ') then
+        igrid6=k1+k2+k3+k4+24*25+24
+     else
+        k5=(ichar(grid6(5:5))-ichar('A'))*25
+        k6=(ichar(grid6(6:6))-ichar('A'))
+        igrid6=k1+k2+k3+k4+k5+k6
+     endif
+     write(c77,1030) n22,igrid6,2,0,n3,i3
 1030 format(b22.22,b25.25,b3.3,b21.21,2b3.3)
   endif
 
@@ -1083,10 +1133,12 @@ subroutine pack77_1(nwords,w,i3,n3,c77)
   if(c1.eq.'+' .or. c1.eq.'-') then
      ir=0
      read(w(nwords),*,err=900) irpt
+     if(irpt.ge.-50 .and. irpt.le.-31) irpt=irpt+101
      irpt=irpt+35
   else if(c2.eq.'R+' .or. c2.eq.'R-') then
      ir=1
-     read(w(nwords)(2:),*) irpt
+     read(w(nwords)(2:),*,err=900) irpt
+     if(irpt.ge.-50 .and. irpt.le.-31) irpt=irpt+101
      irpt=irpt+35
   else if(trim(w(nwords)).eq.'RRR') then
      ir=0
@@ -1153,7 +1205,7 @@ subroutine pack77_3(nwords,w,i3,n3,c77)
 !             - DX:     rpt serial          R 559 0013
 ! Example message:  TU; W9XYZ K1ABC R 579 MA           1 28 28 1 3 13   74
   
-  parameter (NUSCAN=65)    !Number of US states and Canadian provinces/territories
+  parameter (NUSCAN=171)    !Number of US states and Canadian provinces/territories
   character*13 w(19)
   character*77 c77
   character*6 bcall_1,bcall_2
@@ -1167,7 +1219,18 @@ subroutine pack77_3(nwords,w,i3,n3,c77)
        "NM ","NY ","NC ","ND ","OH ","OK ","OR ","PA ","RI ","SC ",  &
        "SD ","TN ","TX ","UT ","VT ","VA ","WA ","WV ","WI ","WY ",  &
        "NB ","NS ","QC ","ON ","MB ","SK ","AB ","BC ","NWT","NF ",  &
-       "LB ","NU ","YT ","PEI","DC "/
+       "LB ","NU ","YT ","PEI","DC ","DR ","FR ","GD ","GR ","OV ",  &
+       "ZH ","ZL ","X01","X02","X03","X04","X05","X06","X07","X08",  &
+       "X09","X10","X11","X12","X13","X14","X15","X16","X17","X18",  &
+       "X19","X20","X21","X22","X23","X24","X25","X26","X27","X28",  &
+       "X29","X30","X31","X32","X33","X34","X35","X36","X37","X38",  &
+       "X39","X40","X41","X42","X43","X44","X45","X46","X47","X48",  &
+       "X49","X50","X51","X52","X53","X54","X55","X56","X57","X58",  &
+       "X59","X60","X61","X62","X63","X64","X65","X66","X67","X68",  &
+       "X69","X70","X71","X72","X73","X74","X75","X76","X77","X78",  &
+       "X79","X80","X81","X82","X83","X84","X85","X86","X87","X88",  &
+       "X89","X90","X91","X92","X93","X94","X95","X96","X97","X98",  &
+       "X99"/
 
   if(w(1)(1:1).eq.'<' .and. w(2)(1:1).eq.'<') go to 900
   if(nwords.eq.4 .or. nwords.eq.5 .or. nwords.eq.6) then
@@ -1243,6 +1306,7 @@ subroutine pack77_4(nwords,w,i3,n3,c77)
      if(call_2(1:1).eq.'<') call_2=w(2)(2:len(trim(w(2)))-1)
      call chkcall(call_1,bcall_1,ok1)
      call chkcall(call_2,bcall_2,ok2)
+     if(call_1.eq.bcall_1 .and. call_2.eq.bcall_2 .and. ok1 .and. ok2) go to 900
      icq=0
      if(trim(w(1)).eq.'CQ' .or. (ok1.and.ok2)) then
         if(trim(w(1)).eq.'CQ' .and. len(trim(w(2))).le.4) go to 900
@@ -1467,44 +1531,96 @@ subroutine add_call_to_recent_calls(callsign)
   return
 end subroutine add_call_to_recent_calls
 
-subroutine to_grid4(n,grid4)
+subroutine to_grid4(n,grid4,ok)
   character*4 grid4
-  
+  logical ok
+
+  ok=.false.
   j1=n/(18*10*10)
+  if (j1.lt.0.or.j1.gt.17) goto 900
   n=n-j1*18*10*10
   j2=n/(10*10)
+  if (j2.lt.0.or.j2.gt.17) goto 900
   n=n-j2*10*10
   j3=n/10
+  if (j3.lt.0.or.j3.gt.9) goto 900
   j4=n-j3*10
+  if (j4.lt.0.or.j4.gt.9) goto 900
   grid4(1:1)=char(j1+ichar('A'))
   grid4(2:2)=char(j2+ichar('A'))
   grid4(3:3)=char(j3+ichar('0'))
   grid4(4:4)=char(j4+ichar('0'))
-  
-  return
+  ok=.true.
+
+900 return
 end subroutine to_grid4
 
-subroutine to_grid6(n,grid6)
+subroutine to_grid6(n,grid6,ok)
   character*6 grid6
+  logical ok
 
+  ok=.false.
   j1=n/(18*10*10*24*24)
+  if (j1.lt.0.or.j1.gt.17) goto 900
   n=n-j1*18*10*10*24*24
   j2=n/(10*10*24*24)
+  if (j2.lt.0.or.j2.gt.17) goto 900
   n=n-j2*10*10*24*24
   j3=n/(10*24*24)
+  if (j3.lt.0.or.j3.gt.9) goto 900
   n=n-j3*10*24*24
   j4=n/(24*24)
+  if (j4.lt.0.or.j4.gt.9) goto 900
   n=n-j4*24*24
   j5=n/24
+  if (j5.lt.0.or.j5.gt.23) goto 900
   j6=n-j5*24
+  if (j6.lt.0.or.j6.gt.23) goto 900
   grid6(1:1)=char(j1+ichar('A'))
   grid6(2:2)=char(j2+ichar('A'))
   grid6(3:3)=char(j3+ichar('0'))
   grid6(4:4)=char(j4+ichar('0'))
   grid6(5:5)=char(j5+ichar('A'))
   grid6(6:6)=char(j6+ichar('A'))  
+  ok=.true.
 
-  return
+900 return
 end subroutine to_grid6
 
+subroutine to_grid(n,grid6,ok)
+  ! 4-, or 6-character grid
+  character*6 grid6
+  logical ok
+
+  ok=.false.
+  j1=n/(18*10*10*25*25)
+  if (j1.lt.0.or.j1.gt.17) goto 900
+  n=n-j1*18*10*10*25*25
+  j2=n/(10*10*25*25)
+  if (j2.lt.0.or.j2.gt.17) goto 900
+  n=n-j2*10*10*25*25
+  j3=n/(10*25*25)
+  if (j3.lt.0.or.j3.gt.9) goto 900
+  n=n-j3*10*25*25
+  j4=n/(25*25)
+  if (j4.lt.0.or.j4.gt.9) goto 900
+  n=n-j4*25*25
+  j5=n/25
+  if (j5.lt.0.or.j5.gt.24) goto 900
+  j6=n-j5*25
+  if (j6.lt.0.or.j6.gt.24) goto 900
+  grid6=''
+  grid6(1:1)=char(j1+ichar('A'))
+  grid6(2:2)=char(j2+ichar('A'))
+  grid6(3:3)=char(j3+ichar('0'))
+  grid6(4:4)=char(j4+ichar('0'))
+  if (j5.ne.24.or.j6.ne.24) then
+     grid6(5:5)=char(j5+ichar('A'))
+     grid6(6:6)=char(j6+ichar('A'))
+  endif
+  ok=.true.
+
+900 return
+end subroutine to_grid
+
 end module packjt77
