@@ -38,12 +38,13 @@ contains
     use iso_c_binding, only: c_bool, c_int
     use timer_module, only: timer
     use shmem, only: shmem_lock, shmem_unlock
+    use ft8_a7
 
     include 'ft8/ft8_params.f90'
 
     class(ft8_decoder), intent(inout) :: this
     procedure(ft8_decode_callback) :: callback
-    parameter (MAXCAND=300,MAX_EARLY=100)
+    parameter (MAXCAND=600,MAX_EARLY=100)
     real*8 tsec,tseq
     real s(NH1,NHSYM)
     real sbase(NH1)
@@ -53,24 +54,47 @@ contains
     logical newdat,lsubtract,ldupe,lrefinedt
     logical*1 ldiskdat
     logical lsubtracted(MAX_EARLY)
-    character*12 mycall12,hiscall12
+    character*12 mycall12,hiscall12,call_1,call_2
+    character*4 grid4
     integer*2 iwave(15*12000)
     integer apsym2(58),aph10(10)
     character datetime*13,msg37*37
-    character*37 allmessages(100)
+    character*37 allmessages(200)
     character*12 ctime
     integer allsnrs(100)
     integer itone(NN)
     integer itone_save(NN,MAX_EARLY)
     real f1_save(MAX_EARLY)
     real xdt_save(MAX_EARLY)
+    data nutc0/-1/
 
-    save s,dd,dd1,ndec_early,itone_save,f1_save,xdt_save,lsubtracted,allmessages
+    save s,dd,dd1,nutc0,ndec_early,itone_save,f1_save,xdt_save,lsubtracted,&
+         allmessages
     
     this%callback => callback
     write(datetime,1001) nutc        !### TEMPORARY ###
 1001 format("000000_",i6.6)
 
+    if(nutc0.eq.-1) then
+       msg0=' '
+       dt0=0.
+       f0=0.
+    endif
+!Added 41==nzhsym to force a reset if the same wav file is processed twice or more in a row,
+!in which case nutc.eq.nutc0 and ndec(jseq,1) doesn't get reset
+    if(nzhsym==41 .or. (nutc.ne.nutc0)) then
+! New UTC.  Move previously saved 'a7' data from k=1 to k=0
+       iz=ndec(jseq,1)
+       dt0(1:iz,jseq,0)  = dt0(1:iz,jseq,1)
+       f0(1:iz,jseq,0)   = f0(1:iz,jseq,1)
+       msg0(1:iz,jseq,0) = msg0(1:iz,jseq,1)
+       ndec(jseq,0)=iz
+       ndec(jseq,1)=0
+       nutc0=nutc
+       dt0(:,jseq,1)=0.
+       f0(:,jseq,1)=0.
+    endif
+
     if(ndepth.eq.1 .and. nzhsym.lt.50) then
        ndec_early=0
        return
@@ -85,6 +109,7 @@ contains
        dd=iwave
        dd1=dd
     endif
+
     if(nzhsym.eq.41) then
        ndecodes=0
        allmessages='                                     '
@@ -92,10 +117,12 @@ contains
     else
        ndecodes=ndec_early
     endif
+
     if(nzhsym.eq.47 .and. ndec_early.eq.0) then
        dd1=dd
        go to 800
     endif
+
     if(nzhsym.eq.47 .and. ndec_early.ge.1) then
        lsubtracted=.false.
        lrefinedt=.true.
@@ -118,7 +145,8 @@ contains
        dd1=dd
        go to 900
     endif
-    if(nzhsym.eq.50 .and. ndec_early.ge.1) then
+
+    if(nzhsym.eq.50 .and. ndec_early.ge.1 .and. .not.nagain) then
        n=47*3456
        dd(1:n)=dd1(1:n)
        dd(n+1:)=iwave(n+1:)
@@ -129,11 +157,13 @@ contains
        enddo
        call timer('sub_ft8c',1)
     endif
+
     ifa=nfa
     ifb=nfb
-    if(nagain) then
-       ifa=nfqso-10
-       ifb=nfqso+10
+    if(nzhsym.eq.50 .and. nagain) then
+       dd=iwave
+       ifa=nfqso-20
+       ifb=nfqso+20
     endif
 
 ! For now:
@@ -141,11 +171,12 @@ contains
 ! ndepth=2: subtraction, 3 passes, bp+osd (no subtract refinement) 
 ! ndepth=3: subtraction, 3 passes, bp+osd
     npass=3
-    if(ndepth.eq.1) npass=1
+    if(ndepth.eq.1) npass=2
     do ipass=1,npass
       newdat=.true.
       syncmin=1.3
       if(ndepth.le.2) syncmin=1.6
+      if(nzhsym.eq.41) syncmin=2.0
       if(ipass.eq.1) then
         lsubtract=.true.
         ndeep=ndepth
@@ -162,7 +193,7 @@ contains
       endif 
       call timer('sync8   ',0)
       maxc=MAXCAND
-      call sync8(dd,ifa,ifb,syncmin,nfqso,maxc,s,candidate,   &
+      call sync8(dd,ifa,ifb,syncmin,nfqso,maxc,nzhsym,candidate,   &
            ncand,sbase)
       call timer('sync8   ',1)
       do icand=1,ncand
@@ -177,7 +208,7 @@ contains
              hiscall12,f1,xdt,xbase,apsym2,aph10,nharderrors,dmin,          &
              nbadcrc,iappass,msg37,xsnr,itone)
         call timer('ft8b    ',1)
-        nsnr=nint(xsnr) 
+        nsnr=nint(xsnr)
         xdt=xdt-0.5
         hd=nharderrors+dmin
         if(nbadcrc.eq.0) then
@@ -197,6 +228,7 @@ contains
               qual=1.0-(nharderrors+dmin)/60.0 ! scale qual to [0.0,1.0]
               if(emedelay.ne.0) xdt=xdt+2.0
               call this%callback(sync,nsnr,xdt,f1,msg37,iaptype,qual)
+              call ft8_a7_save(nutc,xdt,f1,msg37)  !Enter decode in table
            endif
         endif
         call timestamp(tsec,tseq,ctime)
@@ -208,7 +240,43 @@ contains
 800 ndec_early=0
    if(nzhsym.lt.50) ndec_early=ndecodes
    
-900 return
+900 continue
+   if(lft8apon .and. ncontest.ne.6 .and. ncontest.ne.7 .and. nzhsym.eq.50 .and. ndec(jseq,0).ge.1) then
+      newdat=.true.
+      do i=1,ndec(jseq,0)
+         if(f0(i,jseq,0).eq.-99.0) exit
+         if(f0(i,jseq,0).eq.-98.0) cycle
+         if(index(msg0(i,jseq,0),'<').ge.1) cycle      !### Temporary ###
+         msg37=msg0(i,jseq,0)
+         i1=index(msg37,' ')
+         i2=index(msg37(i1+1:),' ') + i1
+         call_1=msg37(1:i1-1)
+         call_2=msg37(i1+1:i2-1)
+         grid4=msg37(i2+1:i2+4)
+         if(grid4.eq.'RR73' .or. index(grid4,'+').gt.0 .or.                      &
+              index(grid4,'-').gt.0) grid4='    '         
+         xdt=dt0(i,jseq,0)
+         f1=f0(i,jseq,0)
+         xbase=10.0**(0.1*(sbase(max(1,nint(f1/3.125)))-40.0))
+         msg37='                                     '
+         call timer('ft8_a7d ',0)
+         call ft8_a7d(dd,newdat,call_1,call_2,grid4,xdt,f1,xbase,nharderrors,   &
+              dmin,msg37,xsnr)
+         call timer('ft8_a7d ',1)
+
+         if(nharderrors.ge.0) then
+            if(associated(this%callback)) then
+               nsnr=xsnr
+               iaptype=7
+               qual=1.0
+               call this%callback(sync,nsnr,xdt,f1,msg37,iaptype,qual)
+               call ft8_a7_save(nutc,xdt,f1,msg37)  !Enter decode in table
+            endif
+         endif
+      enddo
+   endif
+
+   return
 end subroutine decode
 
 subroutine timestamp(tsec,tseq,ctime)
