@@ -15,9 +15,11 @@
 #include <QScrollBar>
 
 #include "Configuration.hpp"
+#include "Decoder/decodedtext.h"
 #include "Network/LotWUsers.hpp"
 #include "models/DecodeHighlightingModel.hpp"
 #include "logbook/logbook.h"
+#include "Logger.hpp"
 
 #include "qt_helpers.hpp"
 #include "moc_displaytext.cpp"
@@ -85,7 +87,7 @@ void DisplayText::mouseDoubleClickEvent(QMouseEvent *e)
 
 void DisplayText::insertLineSpacer(QString const& line)
 {
-  appendText (line, "#d3d3d3");
+  insertText (line, "#d3d3d3");
 }
 
 namespace
@@ -122,11 +124,11 @@ namespace
   }
 }
 
-void DisplayText::appendText(QString const& text, QColor bg, QColor fg
-                             , QString const& call1, QString const& call2)
+void DisplayText::insertText(QString const& text, QColor bg, QColor fg
+                             , QString const& call1, QString const& call2, QTextCursor::MoveOperation location)
 {
   auto cursor = textCursor ();
-  cursor.movePosition (QTextCursor::End);
+  cursor.movePosition (location);
   auto block_format = cursor.blockFormat ();
   auto format = cursor.blockCharFormat ();
   format.setFont (char_font_);
@@ -211,9 +213,25 @@ void DisplayText::appendText(QString const& text, QColor bg, QColor fg
 
 void DisplayText::extend_vertical_scrollbar (int min, int max)
 {
-  if (high_volume_
-      && m_config && m_config->decodes_from_top ())
+  static int mod_last;
+  static int height;
+  if (high_volume_ && m_config && m_config->decodes_from_top ())
     {
+      auto m = modified_vertical_scrollbar_max_;
+      if (m != mod_last) { height = m - mod_last;mod_last = m; }
+      //auto vp_margins2 = viewportMargins ();
+      if (height == 0 && m > viewport()->height()) height =  abs( - viewport()->height());
+      //LOG_INFO ("scrollbar min=" << min << " max="  << max << " mod=" << modified_vertical_scrollbar_max_ << " height=" << viewport()->height() << " top=" << vp_margins2.top() << " bottom=" << vp_margins2.bottom()) << " height=" << height << " mod_last=" << mod_last;
+      if (max > 60000)
+        {
+          QString tmp = toPlainText();
+          while (tmp != NULL && tmp.length() > 100 &&  max > 50000)
+          {
+            tmp.remove(0, tmp.indexOf("\n")+1);
+            max -= height;
+          }
+          setPlainText(tmp);
+        }
       if (max && max != modified_vertical_scrollbar_max_)
         {
           auto vp_margins = viewportMargins ();
@@ -241,10 +259,9 @@ void DisplayText::new_period ()
 
 QString DisplayText::appendWorkedB4 (QString message, QString call, QString const& grid,
                                      QColor * bg, QColor * fg, LogBook const& logBook,
-                                     QString const& currentBand, QString const& currentMode)
+                                     QString const& currentBand, QString const& currentMode,
+                                     QString extra)
 {
-  // allow for seconds
-  int padding {message.indexOf (" ") > 4 ? 2 : 0};
   QString countryName;
   bool callB4;
   bool callB4onBand;
@@ -277,8 +294,9 @@ QString DisplayText::appendWorkedB4 (QString message, QString call, QString cons
     gridB4onBand=true;
   }
 
+  if(callB4onBand) m_points=0;
+
   message = message.trimmed ();
-  QString appendage;
 
   highlight_types types;
   // no shortcuts here as some types may be disabled
@@ -329,20 +347,20 @@ QString DisplayText::appendWorkedB4 (QString message, QString call, QString cons
     {
     case Highlight::Continent:
     case Highlight::ContinentBand:
-      appendage = AD1CCty::continent (looked_up.continent);
+      extra += AD1CCty::continent (looked_up.continent);
       break;
     case Highlight::CQZone:
     case Highlight::CQZoneBand:
-      appendage = QString {"CQ Zone %1"}.arg (looked_up.CQ_zone);
+      extra += QString {"CQ Zone %1"}.arg (looked_up.CQ_zone);
       break;
     case Highlight::ITUZone:
     case Highlight::ITUZoneBand:
-      appendage = QString {"ITU Zone %1"}.arg (looked_up.ITU_zone);
+      extra += QString {"ITU Zone %1"}.arg (looked_up.ITU_zone);
       break;
     default:
       if (m_bPrincipalPrefix)
         {
-          appendage = looked_up.primary_prefix;
+          extra += looked_up.primary_prefix;
         }
       else
         {
@@ -368,31 +386,52 @@ QString DisplayText::appendWorkedB4 (QString message, QString call, QString cons
           countryName.replace ("European", "EU");
           countryName.replace ("African", "AF");
 
-          appendage += countryName;
+          extra += countryName;
         }
     }
     m_CQPriority=DecodeHighlightingModel::highlight_name(top_highlight);
 
-  // use a nbsp to save the start of appended text so we can find
-  // it again later, align appended data at a fixed column if
-  // there is space otherwise let it float to the right
-  int space_count {40 + padding - message.size ()};
-  if (space_count > 0) {
-    message += QString {space_count, QChar {' '}};
+    if(((m_points == 00) or (m_points == -1)) and m_bDisplayPoints) return message;
+    return leftJustifyAppendage (message, extra);
+}
+
+QString DisplayText::leftJustifyAppendage (QString message, QString const& appendage0) const
+{
+  QString appendage=appendage0;
+  if(m_bDisplayPoints and (m_points>0)) {
+    appendage=" " + QString::number(m_points);
+    if(m_points<10) appendage=" " + appendage;
   }
-  message += QChar::Nbsp + appendage;
+  if (appendage.size ())
+    {
+      // allow for seconds
+      int padding {message.indexOf (" ") > 4 ? 2 : 0};
+
+      // use a nbsp to save the start of appended text so we can find
+      // it again later, align appended data at a fixed column if
+      // there is space otherwise let it float to the right
+      int space_count {40 + padding - message.size ()};
+      if (space_count > 0) {
+        message += QString {space_count, QChar {' '}};
+      }
+      message += QChar::Nbsp + appendage;
+    }
   return message;
 }
 
 void DisplayText::displayDecodedText(DecodedText const& decodedText, QString const& myCall,
                                      QString const& mode,
                                      bool displayDXCCEntity, LogBook const& logBook,
-                                     QString const& currentBand, bool ppfx, bool bCQonly)
+                                     QString const& currentBand, bool ppfx, bool bCQonly,
+                                     bool haveFSpread, float fSpread, bool bDisplayPoints, int points)
 {
+  m_points=points;
+  m_bDisplayPoints=bDisplayPoints;
   m_bPrincipalPrefix=ppfx;
   QColor bg;
   QColor fg;
   bool CQcall = false;
+  auto is_73 = decodedText.messageWords().filter (QRegularExpression {"^(73|RR73)$"}).size();
   if (decodedText.string ().contains (" CQ ")
       || decodedText.string ().contains (" CQDX ")
       || decodedText.string ().contains (" QRZ "))
@@ -402,17 +441,6 @@ void DisplayText::displayDecodedText(DecodedText const& decodedText, QString con
   else
     {
       if (bCQonly) return;
-      if (myCall != "" && (decodedText.indexOf (" " + myCall + " ") >= 0
-                           or decodedText.indexOf (" " + myCall + "/") >= 0
-                           or decodedText.indexOf ("<" + myCall + "/") >= 0
-                           or decodedText.indexOf ("/" + myCall + " ") >= 0
-                           or decodedText.indexOf ("/" + myCall + ">") >= 0
-                           or decodedText.indexOf ("<" + myCall + " ") >= 0
-                           or decodedText.indexOf ("<" + myCall + ">") >= 0
-                           or decodedText.indexOf (" " + myCall + ">") >= 0)) {
-        highlight_types types {Highlight::MyCall};
-        set_colours (m_config, &bg, &fg, types);
-      }
     }
   auto message = decodedText.string();
   QString dxCall;
@@ -420,24 +448,32 @@ void DisplayText::displayDecodedText(DecodedText const& decodedText, QString con
   decodedText.deCallAndGrid (/*out*/ dxCall, dxGrid);
   QRegularExpression grid_regexp {"\\A(?![Rr]{2}73)[A-Ra-r]{2}[0-9]{2}([A-Xa-x]{2}){0,1}\\z"};
   if(!dxGrid.contains(grid_regexp)) dxGrid="";
-  message = message.left (message.indexOf (QChar::Nbsp)); // strip appended info
+  message = message.left (message.indexOf (QChar::Nbsp)).trimmed (); // strip appended info
+  QString extra;
+  if (haveFSpread)
+    {
+      extra += QString {"%1"}.arg (fSpread, 5, 'f', fSpread < 0.95 ? 3 : 2) + QChar {' '};
+    }
+  auto ap_pos = message.lastIndexOf (QRegularExpression {R"((?:\?\s)?(?:a[0-9]|q[0-9][0-9*]?)$)"});
+  if (ap_pos >= 0)
+    {
+      extra += message.mid (ap_pos) + QChar {' '};
+      message = message.left (ap_pos).trimmed ();
+    }
   m_CQPriority="";
-  if (CQcall)
+  if (CQcall || (is_73 && (m_config->highlight_73 ())))
     {
       if (displayDXCCEntity)
         {
           // if enabled add the DXCC entity and B4 status to the end of the
           // preformated text line t1
           auto currentMode = mode;
-          if ("JT9+JT65" == mode)
-            {
-              currentMode = decodedText.isJT65 () ? "JT65" : "JT9";
-            }
-          message = appendWorkedB4 (message, decodedText.CQersCall(), dxGrid, &bg, &fg
-                                    , logBook, currentBand, currentMode);
+          message = appendWorkedB4 (message, dxCall, dxGrid, &bg, &fg
+                                    , logBook, currentBand, currentMode, extra);
         }
       else
         {
+          message = leftJustifyAppendage (message, extra);
           highlight_types types {Highlight::CQ};
           if (m_config && m_config->lotw_users ().user (decodedText.CQersCall()))
             {
@@ -446,23 +482,44 @@ void DisplayText::displayDecodedText(DecodedText const& decodedText, QString con
           set_colours (m_config, &bg, &fg, types);
         }
     }
+  else
+    {
+      message = leftJustifyAppendage (message, extra);
+    }
 
-  appendText (message.trimmed (), bg, fg, decodedText.call (), dxCall);
+  if (myCall.size ())
+    {
+      QString regexp {"[ <]" + myCall + "[ >]"};
+      if (Radio::is_compound_callsign (myCall))
+        {
+          regexp = "(?:" + regexp + "|[ <]" + Radio::base_callsign (myCall) + "[ >])";
+        }
+      if ((decodedText.clean_string () + " ").contains (QRegularExpression {regexp}))
+        {
+          highlight_types types {Highlight::MyCall};
+          set_colours (m_config, &bg, &fg, types);
+        }
+    }
+
+  insertText (message.trimmed (), bg, fg, decodedText.call (), dxCall);
 }
 
 
-void DisplayText::displayTransmittedText(QString text, QString modeTx, qint32 txFreq,bool bFastMode)
+void DisplayText::displayTransmittedText(QString text, QString modeTx, qint32 txFreq,
+                                         bool bFastMode, double TRperiod)
 {
     QString t1=" @  ";
     if(modeTx=="FT4") t1=" +  ";
     if(modeTx=="FT8") t1=" ~  ";
     if(modeTx=="JT4") t1=" $  ";
+    if(modeTx=="Q65") t1=" :  ";
     if(modeTx=="JT65") t1=" #  ";
     if(modeTx=="MSK144") t1=" &  ";
+    if(modeTx=="FST4") t1=" `  ";
     QString t2;
     t2 = t2.asprintf("%4d",txFreq);
     QString t;
-    if(bFastMode or modeTx=="FT8" or modeTx=="FT4") {
+    if(bFastMode or modeTx=="FT8" or modeTx=="FT4" or (TRperiod<60)) {
       t = QDateTime::currentDateTimeUtc().toString("hhmmss") + \
         "  Tx      " + t2 + t1 + text;
     } else if(modeTx.mid(0,6)=="FT8fox") {
@@ -476,18 +533,19 @@ void DisplayText::displayTransmittedText(QString text, QString modeTx, qint32 tx
     QColor fg;
     highlight_types types {Highlight::Tx};
     set_colours (m_config, &bg, &fg, types);
-    appendText (t, bg, fg);
+    insertText (t, bg, fg);
 }
 
 void DisplayText::displayQSY(QString text)
 {
   QString t = QDateTime::currentDateTimeUtc().toString("hhmmss") + "            " + text;
-  appendText (t, "hotpink");
+  insertText (t, "hotpink");
 }
 
-void DisplayText::displayFoxToBeCalled(QString t, QColor bg, QColor fg)
+void DisplayText::displayHoundToBeCalled(QString t, bool bAtTop, QColor bg, QColor fg)
 {
-  appendText (t, bg, fg);
+  if (bAtTop)  t = t + "\n"; // need a newline when insertion at top
+  insertText(t, bg, fg, "", "", bAtTop ? QTextCursor::Start : QTextCursor::End);
 }
 
 namespace
