@@ -3,6 +3,7 @@
 #define MAINWINDOW_H
 
 #include <QMainWindow>
+#include <QByteArray>
 #include <QString>
 #include <QStringList>
 #include <QLabel>
@@ -21,9 +22,12 @@
 #include <QPointer>
 #include <QSet>
 #include <QVector>
+#include <QQueue>
 #include <QFuture>
 #include <QFutureWatcher>
 
+#include "MultiGeometryWidget.hpp"
+#include "NonInheritingProcess.hpp"
 #include "Audio/AudioDevice.hpp"
 #include "commons.h"
 #include "Radio.hpp"
@@ -33,7 +37,7 @@
 #include "WSPR/WSPRBandHopping.hpp"
 #include "Transceiver/Transceiver.hpp"
 #include "DisplayManual.hpp"
-#include "Network/psk_reporter.h"
+#include "Network/PSKReporter.hpp"
 #include "logbook/logbook.h"
 #include "astro.h"
 #include "MessageBox.hpp"
@@ -43,18 +47,17 @@
 #define NUM_JT65_SYMBOLS 126               //63 data + 63 sync
 #define NUM_JT9_SYMBOLS 85                 //69 data + 16 sync
 #define NUM_WSPR_SYMBOLS 162               //(50+31)*2, embedded sync
-#define NUM_WSPR_LF_SYMBOLS 412            //300 data + 109 sync + 3 ramp
-#define NUM_ISCAT_SYMBOLS 1291             //30*11025/256
 #define NUM_MSK144_SYMBOLS 144             //s8 + d48 + s8 + d80
-#define NUM_QRA64_SYMBOLS 84               //63 data + 21 sync
+#define NUM_Q65_SYMBOLS 85                 //63 data + 22 sync
 #define NUM_FT8_SYMBOLS 79
 #define NUM_FT4_SYMBOLS 105
+#define NUM_FST4_SYMBOLS 160             //240/2 data + 5*8 sync
 #define NUM_CW_SYMBOLS 250
+#define MAX_NUM_SYMBOLS 250
 #define TX_SAMPLE_RATE 48000
-#define N_WIDGETS 33
 #define NRING 3456000
 
-extern int volatile itone[NUM_ISCAT_SYMBOLS];   //Audio tones for all Tx symbols
+extern int volatile itone[MAX_NUM_SYMBOLS];   //Audio tones for all Tx symbols
 extern int volatile icw[NUM_CW_SYMBOLS];	    //Dits for CW ID
 
 //--------------------------------------------------------------- MainWindow
@@ -62,6 +65,7 @@ namespace Ui {
   class MainWindow;
 }
 
+class QProcessEnvironment;
 class QSharedMemory;
 class QSplashScreen;
 class QSettings;
@@ -74,6 +78,7 @@ class WideGraph;
 class LogQSO;
 class Transceiver;
 class MessageAveraging;
+class ActiveStations;
 class FoxLogWindow;
 class CabrilloLogWindow;
 class ColorHighlighting;
@@ -91,7 +96,8 @@ class MultiSettings;
 class EqualizationToolsDialog;
 class DecodedText;
 
-class MainWindow : public QMainWindow
+class MainWindow
+  : public MultiGeometryWidget<3, QMainWindow>
 {
   Q_OBJECT;
 
@@ -99,10 +105,11 @@ public:
   using Frequency = Radio::Frequency;
   using FrequencyDelta = Radio::FrequencyDelta;
   using Mode = Modes::Mode;
+  using SpecOp = Configuration::SpecialOperatingActivity;
 
   explicit MainWindow(QDir const& temp_directory, bool multiple, MultiSettings *,
                       QSharedMemory *shdmem, unsigned downSampleFactor,
-                      QSplashScreen *,
+                      QSplashScreen *, QProcessEnvironment const&,
                       QWidget *parent = nullptr);
   ~MainWindow();
 
@@ -120,6 +127,7 @@ public slots:
   void doubleClickOnCall (Qt::KeyboardModifiers);
   void doubleClickOnCall2(Qt::KeyboardModifiers);
   void doubleClickOnFoxQueue(Qt::KeyboardModifiers);
+  void doubleClickOnFoxInProgress(Qt::KeyboardModifiers modifiers);
   void readFromStdout();
   void p1ReadFromStdout();
   void setXIT(int n, Frequency base = 0u);
@@ -127,7 +135,8 @@ public slots:
   void msgAvgDecode2();
   void fastPick(int x0, int x1, int y);
 
-protected:
+private:
+  void change_layout (std::size_t) override;
   void keyPressEvent (QKeyEvent *) override;
   void closeEvent(QCloseEvent *) override;
   void childEvent(QChildEvent *) override;
@@ -135,6 +144,12 @@ protected:
 
 private slots:
   void initialize_fonts ();
+  void on_houndButton_clicked(bool checked);
+  void on_ft8Button_clicked();
+  void on_ft4Button_clicked();
+  void on_msk144Button_clicked();
+  void on_q65Button_clicked();
+  void on_jt65Button_clicked();
   void on_tx1_editingFinished();
   void on_tx2_editingFinished();
   void on_tx3_editingFinished();
@@ -149,7 +164,9 @@ private slots:
   void on_stopButton_clicked();
   void on_actionRelease_Notes_triggered ();
   void on_actionFT8_DXpedition_Mode_User_Guide_triggered();
-  void on_actionQuick_Start_Guide_v2_triggered();
+  void on_actionQSG_FST4_triggered();
+  void on_actionQSG_Q65_triggered();
+  void on_actionQSG_X250_M3_triggered();
   void on_actionOnline_User_Guide_triggered();
   void on_actionLocal_User_Guide_triggered();
   void on_actionWide_Waterfall_triggered();
@@ -164,6 +181,7 @@ private slots:
   void on_actionSpecial_mouse_commands_triggered();
   void on_actionSolve_FreqCal_triggered();
   void on_actionCopyright_Notice_triggered();
+  void on_actionSWL_Mode_triggered (bool checked);
   void on_DecodeButton_clicked (bool);
   void decode();
   void decodeBusy(bool b);
@@ -195,15 +213,17 @@ private slots:
   void on_addButton_clicked();
   void on_dxCallEntry_textChanged (QString const&);
   void on_dxGridEntry_textChanged (QString const&);
+  void on_dxCallEntry_editingFinished();
   void on_dxCallEntry_returnPressed ();
   void on_genStdMsgsPushButton_clicked();
   void on_logQSOButton_clicked();
   void on_actionJT9_triggered();
   void on_actionJT65_triggered();
-  void on_actionJT9_JT65_triggered();
   void on_actionJT4_triggered();
   void on_actionFT4_triggered();
   void on_actionFT8_triggered();
+  void on_actionFST4_triggered();
+  void on_actionFST4W_triggered();
   void on_TxFreqSpinBox_valueChanged(int arg1);
   void on_actionSave_decoded_triggered();
   void on_actionQuickDecode_toggled (bool);
@@ -219,15 +239,6 @@ private slots:
   void startP1();
   void stopTx();
   void stopTx2();
-  void on_pbCallCQ_clicked();
-  void on_pbAnswerCaller_clicked();
-  void on_pbSendRRR_clicked();
-  void on_pbAnswerCQ_clicked();
-  void on_pbSendReport_clicked();
-  void on_pbSend73_clicked();
-  void on_rbGenMsg_clicked(bool checked);
-  void on_rbFreeText_clicked(bool checked);
-  void on_freeTextMsg_currentTextChanged (QString const&);
   void on_rptSpinBox_valueChanged(int n);
   void killFile();
   void on_tuneButton_clicked (bool);
@@ -240,11 +251,11 @@ private slots:
                   , QString const& name, QDateTime const& QSO_date_on, QString const& operator_call
                   , QString const& my_call, QString const& my_grid
                   , QString const& exchange_sent, QString const& exchange_rcvd
-                  , QByteArray const& ADIF);
+                  , QString const& propmode, QByteArray const& ADIF);
   void on_bandComboBox_currentIndexChanged (int index);
+  void on_bandComboBox_editTextChanged (QString const& text);
   void on_bandComboBox_activated (int index);
   void on_readFreq_clicked();
-  void on_pbTxMode_clicked();
   void on_RxFreqSpinBox_valueChanged(int n);
   void on_outAttenuation_valueChanged (int);
   void rigOpen ();
@@ -254,16 +265,19 @@ private slots:
   void on_actionShort_list_of_add_on_prefixes_and_suffixes_triggered();
   void band_changed (Frequency);
   void monitor (bool);
+  void end_tuning ();
   void stop_tuning ();
   void stopTuneATU();
   void auto_tx_mode(bool);
   void on_actionMessage_averaging_triggered();
+  void on_actionActiveStations_triggered();
   void on_contest_log_action_triggered ();
   void on_fox_log_action_triggered ();
   void on_actionColors_triggered();
   void on_actionInclude_averaging_toggled (bool);
   void on_actionInclude_correlation_toggled (bool);
   void on_actionEnable_AP_DXcall_toggled (bool);
+  void on_actionAuto_Clear_Avg_toggled (bool);
   void VHF_features_enabled(bool b);
   void on_sbSubmode_valueChanged(int n);
   void on_cbShMsgs_toggled(bool b);
@@ -271,53 +285,58 @@ private slots:
   void on_cbTx6_toggled(bool b);
   void on_cbMenus_toggled(bool b);
   void on_cbCQonly_toggled(bool b);
-  void on_cbFirst_toggled(bool b);
   void on_cbAutoSeq_toggled(bool b);
   void networkError (QString const&);
   void on_ClrAvgButton_clicked();
   void on_actionWSPR_triggered();
-  void on_actionWSPR_LF_triggered();
   void on_syncSpinBox_valueChanged(int n);
   void on_TxPowerComboBox_currentIndexChanged(int);
   void on_sbTxPercent_valueChanged(int n);
   void on_cbUploadWSPR_Spots_toggled(bool b);
   void WSPR_config(bool b);
-  void uploadSpots();
+  void uploadWSPRSpots (bool direct_post = false, QString const& decode_text = QString {});
   void TxAgain();
-  void uploadResponse(QString response);
+  void uploadResponse(QString const& response);
   void on_WSPRfreqSpinBox_valueChanged(int n);
+  void on_sbFST4W_RxFreq_valueChanged(int n);
+  void on_sbFST4W_FTol_valueChanged(int n);
   void on_pbTxNext_clicked(bool b);
   void on_actionEcho_Graph_triggered();
   void on_actionEcho_triggered();
-  void on_actionISCAT_triggered();
   void on_actionFast_Graph_triggered();
   void fast_decode_done();
   void on_actionMeasure_reference_spectrum_triggered();
   void on_actionErase_reference_spectrum_triggered();
   void on_actionMeasure_phase_response_triggered();
   void on_sbTR_valueChanged (int);
+  void on_sbTR_FST4W_valueChanged (int);
   void on_sbFtol_valueChanged (int);
   void on_cbFast9_clicked(bool b);
   void on_sbCQTxFreq_valueChanged(int n);
   void on_cbCQTx_toggled(bool b);
   void on_actionMSK144_triggered();
-  void on_actionQRA64_triggered();
+  void on_actionQ65_triggered();
   void on_actionFreqCal_triggered();
   void splash_done ();
   void on_measure_check_box_stateChanged (int);
   void on_sbNlist_valueChanged(int n);
   void on_sbNslots_valueChanged(int n);
   void on_sbMax_dB_valueChanged(int n);
+  void on_sbF_Low_valueChanged(int n);
+  void on_sbF_High_valueChanged(int n);
+  void chk_FST4_freq_range();
   void on_pbFoxReset_clicked();
   void on_comboBoxHoundSort_activated (int index);
   void not_GA_warning_message ();
   void checkMSK144ContestType();
   void on_pbBestSP_clicked();
-  int  setTxMsg(int n);
+  void on_RoundRobin_currentTextChanged(QString text);
+  void  setTxMsg(int n);
   bool stdCall(QString const& w);
   void remote_configure (QString const& mode, quint32 frequency_tolerance, QString const& submode
                          , bool fast_mode, quint32 tr_period, quint32 rx_df, QString const& dx_call
                          , QString const& dx_grid, bool generate_messages);
+  void callSandP2(int nline);
 
 private:
   Q_SIGNAL void initializeAudioOutputStream (QAudioDeviceInfo,
@@ -335,24 +354,27 @@ private:
   Q_SIGNAL void transmitFrequency (double) const;
   Q_SIGNAL void endTransmitMessage (bool quick = false) const;
   Q_SIGNAL void tune (bool = true) const;
-  Q_SIGNAL void sendMessage (unsigned symbolsLength, double framesPerSymbol,
-      double frequency, double toneSpacing,
+  Q_SIGNAL void sendMessage (QString mode, unsigned symbolsLength,
+      double framesPerSymbol, double frequency, double toneSpacing,
       SoundOutput *, AudioDevice::Channel = AudioDevice::Mono,
       bool synchronize = true, bool fastMode = false, double dBSNR = 99.,
                              int TRperiod=60) const;
   Q_SIGNAL void outAttenuationChanged (qreal) const;
   Q_SIGNAL void toggleShorthand () const;
+  Q_SIGNAL void reset_audio_input_stream (bool report_dropped_frames) const;
 
 private:
   void set_mode (QString const& mode);
   void astroUpdate ();
   void writeAllTxt(QString message);
   void auto_sequence (DecodedText const& message, unsigned start_tolerance, unsigned stop_tolerance);
-  void hideMenus(bool b);
+  void trim_view (bool b);
   void foxTest();
   void setColorHighlighting();
   void chkFT4();
+  bool elide_tx1_not_allowed () const;
 
+  QProcessEnvironment const& m_env;
   NetworkAccessManager m_network_manager;
   bool m_valid;
   QSplashScreen * m_splash;
@@ -380,6 +402,7 @@ private:
   QScopedPointer<HelpTextWindow> m_prefixes;
   QScopedPointer<HelpTextWindow> m_mouseCmnds;
   QScopedPointer<MessageAveraging> m_msgAvgWidget;
+  QScopedPointer<ActiveStations> m_ActiveStationsWidget;
   QScopedPointer<FoxLogWindow> m_foxLogWindow;
   QScopedPointer<CabrilloLogWindow> m_contestLogWindow;
   QScopedPointer<ColorHighlighting> m_colorHighlighting;
@@ -394,21 +417,27 @@ private:
   SoundInput * m_soundInput;
   Modulator * m_modulator;
   SoundOutput * m_soundOutput;
+  int m_rx_audio_buffer_frames;
+  int m_tx_audio_buffer_frames;
   QThread m_audioThread;
 
   qint64  m_msErase;
   qint64  m_secBandChanged;
   qint64  m_freqMoon;
-  qint64  m_msec0;
   qint64  m_fullFoxCallTime;
 
   Frequency m_freqNominal;
+  Frequency m_freqNominalPeriod;
   Frequency m_freqTxNominal;
   Astro::Correction m_astroCorrection;
+  bool m_reverse_Doppler;
 
-  double  m_s6;
   double  m_tRemaining;
   double  m_TRperiod;
+  double  m_fSpread;
+  double  m_s6;
+  double  m_fDither;
+  double  m_fAudioShift;
 
   float   m_DTtol;
   float   m_t0;
@@ -438,16 +467,17 @@ private:
   qint32  m_secID;
   qint32  m_idleMinutes;
   qint32  m_nSubMode;
+  qint32  m_nSubMode_Q65;
+  qint32  m_nSubMode_JT65;
+  qint32  m_nSubMode_JT4;
   qint32  m_nclearave;
   qint32  m_minSync;
   qint32  m_dBm;
-  qint32  m_pctx;
-  qint32  m_nseq;
   qint32  m_nWSPRdecodes;
   qint32  m_k0;
   qint32  m_kdone;
   qint32  m_nPick;
-  FrequencyList_v2::const_iterator m_frequency_list_fcal_iter;
+  FrequencyList_v2_101::const_iterator m_frequency_list_fcal_iter;
   qint32  m_nTx73;
   qint32  m_UTCdisk;
   qint32  m_wait;
@@ -472,6 +502,12 @@ private:
   qint32  m_earlyDecode=41;
   qint32  m_earlyDecode2=47;
   qint32  m_nDecodes=0;
+  qint32  m_maxPoints=-1;
+  qint32  m_latestDecodeTime=-1;
+  qint32  m_points=-99;
+  qint32  m_score=0;
+  qint32  m_fDop=0;
+  qint32  m_echoSec0=0;
 
   bool    m_btxok;		//True if OK to transmit
   bool    m_diskData;
@@ -498,9 +534,8 @@ private:
   QString m_tBlankLine;
   bool    m_bShMsgs;
   bool    m_bSWL;
-  bool    m_uploadSpots;
+  bool    m_uploadWSPRSpots;
   bool    m_uploading;
-  bool    m_txNext;
   bool    m_grid6;
   bool    m_tuneup;
   bool    m_bTxTime;
@@ -527,6 +562,10 @@ private:
   bool    m_bWarnedSplit=false;
   bool    m_bTUmsg;
   bool    m_bBestSPArmed=false;
+  bool    m_bOK_to_chk=false;
+  bool    m_bSentReport=false;
+
+  SpecOp  m_specOp;
 
   enum
     {
@@ -571,9 +610,9 @@ private:
   QFutureWatcher<void> watcher3;
   QFutureWatcher<QString> m_saveWAVWatcher;
 
-  QProcess proc_jt9;
-  QProcess p1;
-  QProcess p3;
+  NonInheritingProcess proc_jt9;
+  NonInheritingProcess p1;
+  NonInheritingProcess p3;
 
   WSPRNet *wsprNet;
 
@@ -599,7 +638,6 @@ private:
   QString m_palette;
   QString m_dateTime;
   QString m_mode;
-  QString m_modeTx;
   QString m_fnameWE;            // save path without extension
   QString m_rpt;
   QString m_nextRpt;
@@ -618,11 +656,15 @@ private:
   QString m_xSent;               //Contest exchange sent
   QString m_xRcvd;               //Contest exchange received
   QString m_currentBand;
+  QString m_currentBandPeriod;
   QString m_nextCall;
   QString m_nextGrid;
   QString m_fileDateTime;
   QString m_inQSOwith;
   QString m_BestCQpriority;
+  QString m_deCall;
+  QString m_deGrid;
+  QString m_ready2call[50];
 
   QSet<QString> m_pfx;
   QSet<QString> m_sfx;
@@ -650,6 +692,34 @@ private:
   };
   QMap<QString,FixupQSO> m_fixupQSO;       //Key = HoundCall, value = info for QSO in progress
 
+  struct ActiveCall
+  {
+    QString grid4;
+    QString bands;
+    qint32 az;
+    qint32 points;
+  };
+  QMap<QString,ActiveCall> m_activeCall;   //Key = callsign, value = grid4, az, points for ARRL_DIGI
+
+  struct RecentCall
+  {
+    qint64 dialFreq;
+    qint32 audioFreq;
+    qint32 snr;
+    qint32 decodeTime;
+    bool   txEven;
+    bool   ready2call;
+  };
+  QMap<QString,RecentCall> m_recentCall;   //Key = callsign, value = snr, dialFreq, audioFreq, decodeTime
+
+  struct ARRL_logged
+  {
+    QDateTime time;
+    QString band;
+    qint32 points;
+  };
+  QList<ARRL_logged> m_arrl_log;
+
   QQueue<QString> m_houndQueue;        //Selected Hounds available for starting a QSO
   QQueue<QString> m_foxQSOinProgress;  //QSOs in progress: Fox has sent a report
   QQueue<qint64>  m_foxRateQueue;
@@ -659,23 +729,22 @@ private:
   QDateTime m_dateTimeSentTx3;
   QDateTime m_dateTimeRcvdRR73;
   QDateTime m_dateTimeBestSP;
+  QDateTime m_dateTimeSeqStart;        //Nominal start time of Rx sequence about to be decoded
 
   QSharedMemory *mem_jt9;
   QString m_QSOText;
-  unsigned m_msAudioOutputBuffered;
-  unsigned m_framesAudioInputBuffered;
   unsigned m_downSampleFactor;
   QThread::Priority m_audioThreadPriority;
   bool m_bandEdited;
   bool m_splitMode;
-  bool m_monitoring;
+  bool m_monitoring=false;
+  bool m_echoRunning=false;
   bool m_tx_when_ready;
   bool m_transmitting;
   bool m_tune;
   bool m_tx_watchdog;           // true when watchdog triggered
   bool m_block_pwr_tooltip;
   bool m_PwrBandSetOK;
-  bool m_bVHFwarned;
   bool m_bDisplayedOnce;
   Frequency m_lastMonitoredFrequency;
   double m_toneSpacing;
@@ -683,7 +752,7 @@ private:
   QProgressDialog m_optimizingProgress;
   QTimer m_heartbeat;
   MessageClient * m_messageClient;
-  PSK_Reporter *psk_Reporter;
+  PSKReporter m_psk_Reporter;
   DisplayManual m_manual;
   QHash<QString, QVariant> m_pwrBandTxMemory; // Remembers power level by band
   QHash<QString, QVariant> m_pwrBandTuneMemory; // Remembers power level by band for tuning
@@ -707,14 +776,14 @@ private:
   void stub();
   void statusChanged();
   void fixStop();
-  bool shortList(QString callsign);
+  bool shortList(QString callsign) const;
   void transmit (double snr = 99.);
   void rigFailure (QString const& reason);
   void pskSetLocal ();
   void pskPost(DecodedText const& decodedtext);
   void displayDialFrequency ();
   void transmitDisplay (bool);
-  void processMessage(DecodedText const&, Qt::KeyboardModifiers = Qt::NoModifier);
+  void processMessage(DecodedText const& message, Qt::KeyboardModifiers = Qt::NoModifier);
   void replyToCQ (QTime, qint32 snr, float delta_time, quint32 delta_frequency, QString const& mode, QString const& message_text, bool low_confidence, quint8 modifiers);
   void locationChange(QString const& location);
   void replayDecodes ();
@@ -726,11 +795,13 @@ private:
   void freqCalStep();
   void setRig (Frequency = 0);  // zero frequency means no change
   void WSPR_history(Frequency dialFreq, int ndecodes);
-  QString WSPR_hhmm(int n);
+  QString beacon_start_time (int n = 0);
+  QString WSPR_message();
   void fast_config(bool b);
   void CQTxFreq();
   void useNextCall();
   void abortQSO();
+  void updateRate();
   void write_all(QString txRx, QString message);
   bool isWorked(int itype, QString key, float fMHz=0, QString="");
 
@@ -749,10 +820,11 @@ private:
   void rm_tb4(QString houndCall);
   void read_wav_file (QString const& fname);
   void decodeDone ();
-  void subProcessFailed (QProcess *, int exit_code, QProcess::ExitStatus);
+  bool subProcessFailed (QProcess *, int exit_code, QProcess::ExitStatus);
   void subProcessError (QProcess *, QProcess::ProcessError);
   void statusUpdate () const;
   void update_watchdog_label ();
+  void invalidate_frequencies_filter ();
   void on_the_minute ();
   void add_child_to_event_filter (QObject *);
   void remove_child_from_event_filter (QObject *);
@@ -760,16 +832,22 @@ private:
   void tx_watchdog (bool triggered);
   qint64  nWidgets(QString t);
   void displayWidgets(qint64 n);
-  void vhfWarning();
   QChar current_submode () const; // returns QChar {0} if submode is not appropriate
   void write_transmit_entry (QString const& file_name);
-  void selectHound(QString t);
+  void selectHound(QString t, bool bTopQueue);
   void houndCallers();
+  void updateFoxQSOsInProgressDisplay();
+  void foxQueueTopCallCommand();
   void foxRxSequencer(QString msg, QString houndCall, QString rptRcvd);
   void foxTxSequencer();
   void foxGenWaveform(int i,QString fm);
   void writeFoxQSO (QString const& msg);
   void to_jt9(qint32 n, qint32 istart, qint32 idone);
+  bool is77BitMode () const;
+  void cease_auto_Tx_after_QSO ();
+  Q_SLOT void ARRL_Digi_Display();
+  void ARRL_Digi_Update(DecodedText dt);
+  void activeWorked(QString call, QString band);
 };
 
 extern int killbyname(const char* progName);
