@@ -7,12 +7,13 @@
 #include <QPen>
 #include <QMouseEvent>
 #include <QDebug>
+#include "qt_helpers.hpp"
 #include "commons.h"
 #include "moc_plotter.cpp"
 #include <fstream>
 #include <iostream>
 
-#define MAX_SCREENSIZE 2048
+#define MAX_SCREENSIZE 8192
 
 extern "C" {
   void flat4_(float swide[], int* iz, int* nflatten);
@@ -48,7 +49,8 @@ CPlotter::CPlotter(QWidget *parent) :                  //CPlotter Constructor
   m_Percent2DScreen0 {0},
   m_rxFreq {1020},
   m_txFreq {0},
-  m_startFreq {0}
+  m_startFreq {0},
+  m_tol {100}
 {
   setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
   setFocusPolicy(Qt::StrongFocus);
@@ -56,6 +58,7 @@ CPlotter::CPlotter(QWidget *parent) :                  //CPlotter Constructor
   setAutoFillBackground(false);
   setAttribute(Qt::WA_OpaquePaintEvent, false);
   setAttribute(Qt::WA_NoSystemBackground, true);
+  setMouseTracking(true);
   m_bReplot=false;
 
   // contextual pop up menu
@@ -97,7 +100,6 @@ void CPlotter::resizeEvent(QResizeEvent* )                    //resizeEvent()
     if(m_bReference) m_h2=m_h-30;
     if(m_h2<1) m_h2=1;
     m_h1=m_h-m_h2;
-//    m_line=0;
     m_2DPixmap = QPixmap(m_Size.width(), m_h2);
     m_2DPixmap.fill(Qt::black);
     m_WaterfallPixmap = QPixmap(m_Size.width(), m_h1);
@@ -108,6 +110,9 @@ void CPlotter::resizeEvent(QResizeEvent* )                    //resizeEvent()
     m_ScalePixmap = QPixmap(m_w,30);
     m_ScalePixmap.fill(Qt::white);
     m_Percent2DScreen0 = m_Percent2DScreen;
+    m_bResized = true;
+    m_vpixperdiv = float(m_h2)/float(VERT_DIVS);
+    m_x=0;
   }
   DrawOverlay();
 }
@@ -127,7 +132,6 @@ void CPlotter::draw(float swide[], bool bScroll, bool bRed)
 {
   if (!m_TRperiod) return;      // not ready to plot yet
   int j,j0;
-  static int ktop=0;
   float y,y2,ymin;
   double fac = sqrt(m_binsPerPixel*m_waterfallAvg/15.0);
   double gain = fac*pow(10.0,0.015*m_plotGain);
@@ -139,7 +143,14 @@ void CPlotter::draw(float swide[], bool bScroll, bool bRed)
 //move current data down one line (must do this before attaching a QPainter object)
   if(bScroll and !m_bReplot) m_WaterfallPixmap.scroll(0,1,0,0,m_w,m_h1);
   QPainter painter1(&m_WaterfallPixmap);
-  m_2DPixmap = m_OverlayPixmap.copy(0,0,m_w,m_h2);
+  if(m_bFirst or bRed or !m_bQ65_Sync or m_mode!=m_mode0
+     or m_bResized or m_rxFreq!=m_rxFreq0) {
+    m_2DPixmap = m_OverlayPixmap.copy(0,0,m_w,m_h2);
+    m_bFirst=false;
+    m_bResized=false;
+    m_rxFreq0=m_rxFreq;
+  }
+  m_mode0=m_mode;
   QPainter painter2D(&m_2DPixmap);
   if(!painter2D.isActive()) return;
   QFont Font("Arial");
@@ -156,12 +167,14 @@ void CPlotter::draw(float swide[], bool bScroll, bool bRed)
   }
   static QPoint LineBuf[MAX_SCREENSIZE];
   static QPoint LineBuf2[MAX_SCREENSIZE];
+  static QPoint LineBuf3[MAX_SCREENSIZE];
+  static QPoint LineBuf4[MAX_SCREENSIZE];
+
   j=0;
   j0=int(m_startFreq/m_fftBinWidth + 0.5);
   int iz=XfromFreq(5000.0);
   int jz=iz*m_binsPerPixel;
   m_fMax=FreqfromX(iz);
-
   if(bScroll and swide[0]<1.e29) {
     flat4_(swide,&iz,&m_Flatten);
     if(!m_bReplot) flat4_(&dec_data.savg[j0],&jz,&m_Flatten);
@@ -184,7 +197,6 @@ void CPlotter::draw(float swide[], bool bScroll, bool bRed)
     if (swide[i]<1.e29) painter1.setPen(g_ColorTbl[y1]);
     painter1.drawPoint(i,m_j);
   }
-
   m_line++;
 
   float y2min=1.e30;
@@ -211,7 +223,7 @@ void CPlotter::draw(float swide[], bool bScroll, bool bRed)
       for(int k=0; k<m_binsPerPixel; k++) {
         sum+=spectra_.syellow[j++];
       }
-      y2=gain2d*sum/m_binsPerPixel + m_plot2dZero;
+      y2=2.0*gain2d*sum/m_binsPerPixel + m_plot2dZero;
     }
 
     if(m_bReference) {                                   //Reference (red)
@@ -222,12 +234,8 @@ void CPlotter::draw(float swide[], bool bScroll, bool bRed)
 
     }
 
-    if(i==iz-1) {
+    if(i==iz-1 and !m_bQ65_Sync and !m_bTotalPower) {
       painter2D.drawPolyline(LineBuf,j);
-      if(m_mode=="QRA64") {
-        painter2D.setPen(Qt::red);
-        painter2D.drawPolyline(LineBuf2,ktop);
-      }
     }
     LineBuf[j].setX(i);
     LineBuf[j].setY(int(0.9*m_h2-y2*m_h2/70.0));
@@ -235,31 +243,34 @@ void CPlotter::draw(float swide[], bool bScroll, bool bRed)
     if(y2>y2max) y2max=y2;
     j++;
   }
-  if(m_bReplot) return;
+  if(m_bReplot and m_mode!="Q65") return;
 
   if(swide[0]>1.0e29) m_line=0;
   if(m_mode=="FT4" and m_line==34) m_line=0;
   if(m_line == painter1.fontMetrics ().height ()) {
     painter1.setPen(Qt::white);
     QString t;
-    qint64 ms = QDateTime::currentMSecsSinceEpoch() % 86400000;
-    int n = fmod(0.001*ms,m_TRperiod);
-    QDateTime t1=QDateTime::currentDateTimeUtc().addSecs(-n);
-    if(m_TRperiod<60.0) {
-      t=t1.toString("hh:mm:ss") + "    " + m_rxBand;
+    if(m_nUTC<0) {
+      auto start = qt_truncate_date_time_to (QDateTime::currentDateTimeUtc(), m_TRperiod * 1e3)
+        .toString (m_TRperiod < 60. ? "hh:mm:ss" : "hh:mm");
+      t = QString {"%1    %2"}.arg (start, m_rxBand);
     } else {
-      t=t1.toString("hh:mm") + "    " + m_rxBand;
+      auto hr = m_nUTC / 10000;
+      auto start = QTime {hr, (m_nUTC - 10000 * hr) / 100, m_nUTC % 100}
+         .toString (m_TRperiod < 60. ? "hh:mm:ss" : "hh:mm");
+      t = QString {"%1    %2"}.arg (start).arg (m_rxBand);
     }
     painter1.drawText (5, painter1.fontMetrics ().ascent (), t);
   }
 
-  if(m_mode=="JT4" or m_mode=="QRA64") {
-    QPen pen3(Qt::yellow);                     //Mark freqs of JT4 single-tone msgs
+  if(m_mode=="JT4" or (m_mode=="Q65" and m_nSubMode>=2)) {
+    DrawOverlay();
+    QPen pen3(Qt::yellow);                     //Mark freqs of JT4/Q65 single-tone msgs
     painter2D.setPen(pen3);
     Font.setWeight(QFont::Bold);
     painter2D.setFont(Font);
     int x1=XfromFreq(m_rxFreq);
-    y=0.2*m_h2;
+    y=0.25*m_h2;
     painter2D.drawText(x1-4,y,"T");
     x1=XfromFreq(m_rxFreq+250);
     painter2D.drawText(x1-4,y,"M");
@@ -269,32 +280,64 @@ void CPlotter::draw(float swide[], bool bScroll, bool bRed)
     painter2D.drawText(x1-4,y,"73");
   }
 
-    if(bRed) {
-      std::ifstream f;
-      f.open(m_redFile.toLatin1());
+  if(bRed and m_bQ65_Sync) {      //Plot the Q65 red/orange sync curves
+    int k=0;
+    int k2=0;
+    std::ifstream f;
+    f.open(m_redFile.toLatin1());
+    if(f) {
+      int x,y;
+      float freq,xdt,smin,smax,sync,sync2;
+      f >> xdt >> smin >> smax;
       if(f) {
-        int x,y;
-        float freq,sync;
-        float slimit=6.0;
-        QPen pen0(Qt::red,1);
-        painter1.setPen(pen0);
         for(int i=0; i<99999; i++) {
-          f >> freq >> sync;
-          if(f.eof()) break;
+          f >> freq >> sync >> sync2;
+          if(!f or f.eof() or k>=MAX_SCREENSIZE or k2>=MAX_SCREENSIZE) break;
           x=XfromFreq(freq);
-          y=(sync-slimit)*3.0;
-          if(y>0) {
-            if(y>15.0) y=15.0;
-            if(x>=0 and x<=m_w) {
-              painter1.setPen(pen0);
-              painter1.drawLine(x,0,x,y);
-            }
+          if(sync > -99.0 and (smin!=0.0 or smax != 0.0)) {
+            y=m_h2*(0.9 - 0.09*gain2d*sync) - m_plot2dZero - 10;
+            LineBuf2[k2].setX(x);                          //Red sync curve
+            LineBuf2[k2].setY(y);
+            k2++;
           }
+          y=m_h2*(0.9 - 0.09*gain2d*sync2) - m_plot2dZero;
+          LineBuf3[k].setX(x);                            //Orange sync curve
+          LineBuf3[k].setY(y);
+          k++;
         }
-        f.close();
       }
-//      m_bDecodeFinished=false;
+      f.close();
+      QPen pen0(Qt::red,2);
+      painter2D.setPen(pen0);
+      if(smin!=0.0 or smax != 0.0) {
+        painter2D.drawPolyline(LineBuf2,k2);
+      }
+      pen0.setColor("orange");
+      painter2D.setPen(pen0);
+      painter2D.drawPolyline(LineBuf3,k);
+      QString t;
+      t = t.asprintf("DT = %6.2f",xdt);
+      painter2D.setPen(Qt::white);
+      Font.setWeight(QFont::Bold);
+      painter2D.setFont(Font);
+      painter2D.drawText(m_w-100,m_h2/2,t);
     }
+  }
+
+  if(m_bTotalPower and m_pdB>1.0) {
+    painter2D.setPen(Qt::green);
+    if(m_x==m_w-1) {
+      for (int i=0; i<m_w-1; i++) {
+        LineBuf4[i].setY(LineBuf4[i+1].y());
+      }
+    }
+    int yy=m_h2 - 0.1*m_vpixperdiv*(m_pdB-20.0);
+    LineBuf4[m_x].setX(m_x);
+    LineBuf4[m_x].setY(yy);
+    if(LineBuf4[m_w-1].y()==0) LineBuf4[m_w-1].setY(yy);
+    painter2D.drawPolyline(LineBuf4,m_x);
+    if(m_x < m_w-1) m_x++;
+  }
 
   update();                                    //trigger a new paintEvent
   m_bScaleOK=true;
@@ -309,6 +352,7 @@ void CPlotter::drawRed(int ia, int ib, float swide[])
 
 void CPlotter::replot()
 {
+  resizeEvent(NULL);
   float swide[m_w];
   m_bReplot=true;
   for(int irow=0; irow<m_h1; irow++) {
@@ -316,6 +360,9 @@ void CPlotter::replot()
     plotsave_(swide,&m_w,&m_h1,&irow);
     draw(swide,false,false);
   }
+  if(m_mode=="Q65" and m_bQ65_Sync) {
+    draw(swide,false,true);
+  }
   update();                                    //trigger a new paintEvent
   m_bReplot=false;
 }
@@ -330,7 +377,6 @@ void CPlotter::DrawOverlay()                   //DrawOverlay()
 
   double df = m_binsPerPixel*m_fftBinWidth;
   QPen penOrange(QColor(255,165,0),3);
-//  QPen penGreen(Qt::green, 3);                 //Mark Tol range with green line
   QPen penGreen(QColor(15,153,105), 3);        //Mark Tol range or BW with dark green line
   QPen penRed(Qt::red, 3);                     //Mark Tx freq with red
   QPainter painter(&m_OverlayPixmap);
@@ -343,7 +389,6 @@ void CPlotter::DrawOverlay()                   //DrawOverlay()
   painter.setBrush(Qt::SolidPattern);
 
   m_fSpan = w*df;
-//  int n=m_fSpan/10;
   m_freqPerDiv=10;
   if(m_fSpan>100) m_freqPerDiv=20;
   if(m_fSpan>250) m_freqPerDiv=50;
@@ -351,25 +396,48 @@ void CPlotter::DrawOverlay()                   //DrawOverlay()
   if(m_fSpan>1000) m_freqPerDiv=200;
   if(m_fSpan>2500) m_freqPerDiv=500;
 
-  pixperdiv = m_freqPerDiv/df;
-  m_hdivs = w*df/m_freqPerDiv + 1.9999;
-
-  float xx0=float(m_startFreq)/float(m_freqPerDiv);
-  xx0=xx0-int(xx0);
-  int x0=xx0*pixperdiv+0.5;
-  for( int i=1; i<m_hdivs; i++) {                  //draw vertical grids
-    x = (int)((float)i*pixperdiv ) - x0;
-    if(x >= 0 and x<=m_w) {
-      painter.setPen(QPen(Qt::white, 1,Qt::DotLine));
-      painter.drawLine(x, 0, x , m_h2);
+  if(!m_bTotalPower) {
+    pixperdiv = m_freqPerDiv/df;
+    m_hdivs = w*df/m_freqPerDiv + 1.9999;
+    float xx0=float(m_startFreq)/float(m_freqPerDiv);
+    xx0=xx0-int(xx0);
+    int x0=xx0*pixperdiv+0.5;
+    for( int i=1; i<m_hdivs; i++) {                 //draw vertical grids
+      x = (int)((float)i*pixperdiv ) - x0;
+      if(x >= 0 and x<=m_w) {
+        painter.setPen(QPen(Qt::white, 1,Qt::DotLine));
+        painter.drawLine(x, 0, x , m_h2);
+      }
     }
   }
 
-  pixperdiv = (float)m_h2 / (float)VERT_DIVS;
   painter.setPen(QPen(Qt::white, 1,Qt::DotLine));
-  for( int i=1; i<VERT_DIVS; i++) {                //draw horizontal grids
-    y = (int)( (float)i*pixperdiv );
-    painter.drawLine(0, y, w, y);
+  if(m_bTotalPower) painter.setPen(QPen(Qt::white, 1,Qt::DashLine));
+  for( int i=1; i<VERT_DIVS; i++) {                 //draw horizontal grids
+    y = int(i*m_vpixperdiv);
+    if(m_bTotalPower) {
+        painter.drawLine(15, y, w, y);
+    } else {
+        painter.drawLine(0, y, w, y);
+    }
+  }
+
+  if(m_bTotalPower) {
+    painter.setPen(QPen(Qt::white));
+    for( int i=1; i<VERT_DIVS; i++) {               //draw horizontal grids
+      y = int(i*m_vpixperdiv);
+      painter.drawText(0,y+5,QString::number(10*(VERT_DIVS-i) + 20));
+    }
+  }
+
+  if(m_bTotalPower and m_h2>100) {
+    painter.setPen(QPen(Qt::white, 1,Qt::DotLine));
+    for( int i=1; i<5*VERT_DIVS; i++) {             //draw horizontal 2 dB grids
+      if(i%5 > 0) {
+        y = int(0.2*i*m_vpixperdiv);
+        painter.drawLine(0, y, w, y);
+      }
+    }
   }
 
   QRect rect0;
@@ -415,7 +483,18 @@ void CPlotter::DrawOverlay()                   //DrawOverlay()
   float bw=9.0*12000.0/m_nsps;               //JT9
   if(m_mode=="FT4") bw=3*12000.0/576.0;      //FT4  ### (3x, or 4x???) ###
   if(m_mode=="FT8") bw=7*12000.0/1920.0;     //FT8
-
+  if(m_mode.startsWith("FST4")) {
+    int h=int(pow(2.0,m_nSubMode));
+    int nsps=800;
+    if(m_TRperiod==30) nsps=1680;
+    if(m_TRperiod==60) nsps=4000;
+    if(m_TRperiod==120) nsps=8400;
+    if(m_TRperiod==300) nsps=21504;
+    if(m_TRperiod==900) nsps=66560;
+    if(m_TRperiod==1800) nsps=134400;
+    float baud=12000.0/nsps;
+    bw=3.0*h*baud;
+  }
   if(m_mode=="JT4") {                        //JT4
     bw=3*11025.0/2520.0;                     //Max tone spacing (3/4 of actual BW)
     if(m_nSubMode==1) bw=2*bw;
@@ -442,7 +521,7 @@ void CPlotter::DrawOverlay()                   //DrawOverlay()
     }
   }
 
-  if(m_modeTx=="JT9" and m_nSubMode>0) {     //JT9
+  if(m_mode=="JT9" and m_nSubMode>0) {     //JT9
     bw=8.0*12000.0/m_nsps;
     if(m_nSubMode==1) bw=2*bw;   //B
     if(m_nSubMode==2) bw=4*bw;   //C
@@ -453,15 +532,17 @@ void CPlotter::DrawOverlay()                   //DrawOverlay()
     if(m_nSubMode==7) bw=128*bw; //H
   }
 
-  if(m_mode=="QRA64") {                      //QRA64
-    bw=63.0*12000.0/m_nsps;
-    if(m_nSubMode==1) bw=2*bw;   //B
-    if(m_nSubMode==2) bw=4*bw;   //C
-    if(m_nSubMode==3) bw=8*bw;   //D
-    if(m_nSubMode==4) bw=16*bw;  //E
+  if(m_mode=="Q65") {                      //Q65
+    int h=int(pow(2.0,m_nSubMode));
+    int nsps=1800;
+    if(m_TRperiod==30) nsps=3600;
+    if(m_TRperiod==60) nsps=7200;
+    if(m_TRperiod==120) nsps=16000;
+    if(m_TRperiod==300) nsps=41472;
+    float baud=12000.0/nsps;
+    bw=65.0*h*baud;
   }
-
-  if(m_modeTx=="JT65") {                     //JT65
+  if(m_mode=="JT65") {                     //JT65
     bw=65.0*11025.0/4096.0;
     if(m_nSubMode==1) bw=2*bw;   //B
     if(m_nSubMode==2) bw=4*bw;   //C
@@ -471,13 +552,13 @@ void CPlotter::DrawOverlay()                   //DrawOverlay()
   if(m_mode=="WSPR") {
     x1=XfromFreq(1400);
     x2=XfromFreq(1600);
-    painter0.drawLine(x1,29,x2,29);
+    painter0.drawLine(x1,26,x2,26);
   }
 
-  if(m_mode=="WSPR-LF") {
-    x1=XfromFreq(1600);
-    x2=XfromFreq(1700);
-    painter0.drawLine(x1,29,x2,29);
+  if(m_mode=="FST4W") {
+    x1=XfromFreq(m_rxFreq-m_tol);
+    x2=XfromFreq(m_rxFreq+m_tol);
+    painter0.drawLine(x1,26,x2,26);
   }
 
   if(m_mode=="FreqCal") {                   //FreqCal
@@ -491,29 +572,38 @@ void CPlotter::DrawOverlay()                   //DrawOverlay()
   int yh=5;
   int yTxTop=12;
   int yRxBottom=yTxTop + 2*yh + 4;
-  if(m_mode=="JT9" or m_mode=="JT65" or m_mode=="JT9+JT65"
-     or m_mode=="QRA64" or m_mode=="FT8" or m_mode=="FT4") {
+  if(m_mode=="JT9" or m_mode=="JT65" or m_mode=="Q65" or m_mode=="FT8"
+     or m_mode=="FT4" or m_mode.startsWith("FST4")) {
 
-    if(m_mode=="QRA64" or (m_mode=="JT65" and m_bVHF)) {
+    if(m_mode=="FST4" and !m_bSingleDecode) {
+      x1=XfromFreq(m_nfa);
+      x2=XfromFreq(m_nfb);
+      painter0.drawLine(x1,25,x1+5,30);   // Mark FST4 F_Low
+      painter0.drawLine(x1,25,x1+5,20);
+      painter0.drawLine(x2,25,x2-5,30);   // Mark FST4 F_High
+      painter0.drawLine(x2,25,x2-5,20);
+    }
+
+    if(m_mode=="Q65" or (m_mode=="JT65" and m_bVHF)) {
       painter0.setPen(penGreen);
       x1=XfromFreq(m_rxFreq-m_tol);
       x2=XfromFreq(m_rxFreq+m_tol);
-      painter0.drawLine(x1,28,x2,28);
+      painter0.drawLine(x1,26,x2,26);
       x1=XfromFreq(m_rxFreq);
-      painter0.drawLine(x1,24,x1,30);
+      painter0.drawLine(x1,20,x1,26);
 
       if(m_mode=="JT65") {
         painter0.setPen(penOrange);
         x3=XfromFreq(m_rxFreq+20.0*bw/65.0);    //RO
-        painter0.drawLine(x3,24,x3,30);
+        painter0.drawLine(x3,20,x3,26);
         x4=XfromFreq(m_rxFreq+30.0*bw/65.0);    //RRR
-        painter0.drawLine(x4,24,x4,30);
+        painter0.drawLine(x4,20,x4,26);
         x5=XfromFreq(m_rxFreq+40.0*bw/65.0);    //73
-        painter0.drawLine(x5,24,x5,30);
+        painter0.drawLine(x5,20,x5,26);
       }
       painter0.setPen(penGreen);
       x6=XfromFreq(m_rxFreq+bw);             //Highest tone
-      painter0.drawLine(x6,24,x6,30);
+      painter0.drawLine(x6,20,x6,26);
 
     } else {
       // Draw the green "goal post"
@@ -523,13 +613,16 @@ void CPlotter::DrawOverlay()                   //DrawOverlay()
       painter0.drawLine(x1,yRxBottom-yh,x1,yRxBottom);
       painter0.drawLine(x1,yRxBottom,x2,yRxBottom);
       painter0.drawLine(x2,yRxBottom-yh,x2,yRxBottom);
-
+      if(m_mode.startsWith("FST4")) {
+        x1=XfromFreq(m_rxFreq-m_tol);
+        x2=XfromFreq(m_rxFreq+m_tol);
+        painter0.drawLine(x1,26,x2,26);   // Mark the Tol range
+      }
     }
   }
 
-  if(m_mode=="JT9" or m_mode=="JT65" or m_mode=="JT9+JT65" or
-     m_mode.mid(0,4)=="WSPR" or m_mode=="QRA64" or m_mode=="FT8"
-     or m_mode=="FT4") {
+  if(m_mode=="JT9" or m_mode=="JT65" or m_mode.mid(0,4)=="WSPR" or m_mode=="Q65"
+     or m_mode=="FT8" or m_mode=="FT4" or m_mode.startsWith("FST4")) {
     painter0.setPen(penRed);
     x1=XfromFreq(m_txFreq);
     x2=XfromFreq(m_txFreq+bw);
@@ -538,27 +631,13 @@ void CPlotter::DrawOverlay()                   //DrawOverlay()
       x1=XfromFreq(m_txFreq-0.5*bw);
       x2=XfromFreq(m_txFreq+0.5*bw);
     }
-    if(m_mode=="WSPR-LF") {
-      bw=3*12000.0/8640.0;                  //WSPR-LF
-      x1=XfromFreq(m_txFreq-0.5*bw);
-      x2=XfromFreq(m_txFreq+0.5*bw);
-    }
     // Draw the red "goal post"
     painter0.drawLine(x1,yTxTop,x1,yTxTop+yh);
     painter0.drawLine(x1,yTxTop,x2,yTxTop);
     painter0.drawLine(x2,yTxTop,x2,yTxTop+yh);
   }
 
-  if(m_mode=="JT9+JT65") {
-    QPen pen2(Qt::blue, 3);                //Mark the JT65 | JT9 divider
-    painter0.setPen(pen2);
-    x1=XfromFreq(m_fMin);
-    if(x1<2) x1=2;
-    x2=x1+30;
-    painter0.drawLine(x1,8,x1,28);
-  }
-
-  if(m_dialFreq>10.13 and m_dialFreq< 10.15 and m_mode.mid(0,4)!="WSPR") {
+  if(m_dialFreq>10.13 and m_dialFreq< 10.15 and m_mode.mid(0,4)!="WSPR" and m_mode!="FST4W") {
     float f1=1.0e6*(10.1401 - m_dialFreq);
     float f2=f1+200.0;
     x1=XfromFreq(f1);
@@ -583,7 +662,6 @@ void CPlotter::MakeFrequencyStrs()                       //MakeFrequencyStrs
 
 int CPlotter::XfromFreq(float f)                               //XfromFreq()
 {
-//  float w = m_WaterfallPixmap.width();
   int x = int(m_w * (f - m_startFreq)/m_fSpan + 0.5);
   if(x<0 ) return 0;
   if(x>m_w) return m_w;
@@ -644,7 +722,9 @@ void CPlotter::setPlot2dZero(int plot2dZero)              //setPlot2dZero
 void CPlotter::setStartFreq(int f)                    //SetStartFreq()
 {
   m_startFreq=f;
+  m_fMax=FreqfromX(XfromFreq(5000.0));
   resizeEvent(NULL);
+  DrawOverlay();
   update();
 }
 
@@ -665,6 +745,7 @@ void CPlotter::setRxRange(int fMin)                           //setRxRange
 void CPlotter::setBinsPerPixel(int n)                         //setBinsPerPixel
 {
   m_binsPerPixel = n;
+  m_fMax=FreqfromX(XfromFreq(5000.0));
   DrawOverlay();                         //Redraw scales and ticks
   update();                              //trigger a new paintEvent}
 }
@@ -688,18 +769,38 @@ void CPlotter::setRxFreq (int x)                               //setRxFreq
 
 int CPlotter::rxFreq() {return m_rxFreq;}                      //rxFreq
 
+void CPlotter::mouseMoveEvent (QMouseEvent * event)
+{
+  int x=event->x();
+  if (!m_bTotalPower){
+      QToolTip::showText(event->globalPos(),QString::number(int(FreqfromX(x))));
+  } else {
+    int y=event->y();
+    float pdB=10.0*(m_h-y)/m_vpixperdiv + 20.0;
+    if(y<(m_h-m_h2)) {
+      QToolTip::showText(event->globalPos(),QString::number(int(FreqfromX(x))));
+    } else {
+      QString t;
+      t=t.asprintf("%4.1f dB",pdB);
+      QToolTip::showText(event->globalPos(),t);
+    }
+  }
+  QWidget::mouseMoveEvent(event);
+}
+
 void CPlotter::mouseReleaseEvent (QMouseEvent * event)
 {
-  if (Qt::LeftButton == event->button ()) {
+  if (Qt::LeftButton == event->button()) {
     int x=event->x();
     if(x<0) x=0;
     if(x>m_Size.width()) x=m_Size.width();
     bool ctrl = (event->modifiers() & Qt::ControlModifier);
     bool shift = (event->modifiers() & Qt::ShiftModifier);
+    if(!shift and m_mode=="FST4W") return;
     int newFreq = int(FreqfromX(x)+0.5);
     int oldTxFreq = m_txFreq;
     int oldRxFreq = m_rxFreq;
-    if (ctrl) {
+    if (ctrl and m_mode!="FST4W") {
       emit setFreq1 (newFreq, newFreq);
     } else if (shift) {
       emit setFreq1 (oldRxFreq, newFreq);
@@ -714,6 +815,7 @@ void CPlotter::mouseReleaseEvent (QMouseEvent * event)
   else {
     event->ignore ();           // let parent handle
   }
+//  replot();                // ### Not needed?  ###
 }
 
 void CPlotter::mouseDoubleClickEvent (QMouseEvent * event)
@@ -723,8 +825,7 @@ void CPlotter::mouseDoubleClickEvent (QMouseEvent * event)
     int n=2;
     if(ctrl) n+=100;
     emit freezeDecode1(n);
-  }
-  else {
+  } else {
     event->ignore ();           // let parent handle
   }
 }
@@ -759,11 +860,6 @@ void CPlotter::setSubMode(int n)                                //setSubMode
   m_nSubMode=n;
 }
 
-void CPlotter::setModeTx(QString modeTx)                        //setModeTx
-{
-  m_modeTx=modeTx;
-}
-
 int CPlotter::Fmax()
 {
   return m_fMax;
@@ -791,9 +887,22 @@ void CPlotter::setFlatten(bool b1, bool b2)
 void CPlotter::setTol(int n)                                 //setTol()
 {
   m_tol=n;
-  DrawOverlay();
 }
 
+void CPlotter::setFST4_FreqRange(int fLow,int fHigh)
+{
+  m_nfa=fLow;
+  m_nfb=fHigh;
+  DrawOverlay();
+  update();
+}
+
+void CPlotter::setSingleDecode(bool b)
+{
+  m_bSingleDecode=b;
+}
+
+
 void CPlotter::setColours(QVector<QColor> const& cl)
 {
   g_ColorTbl = cl;
@@ -814,3 +923,18 @@ void CPlotter::setRedFile(QString fRed)
 {
   m_redFile=fRed;
 }
+
+void CPlotter::setDiskUTC(int nutc)
+{
+  m_nUTC=nutc;
+}
+
+void CPlotter::drawTotalPower(float pdB)
+{
+  m_pdB=pdB;
+}
+
+void CPlotter::restartTotalPower()
+{
+  m_x=0;
+}
